/*
 * <3BSD notice header>
 *
 * Copyright (C) 2003-2008 Raritan, Inc. <http://www.raritan.com/>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 * 
 *     * Neither the name of [original copyright holder] nor the names of
 *       its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written
 *       permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * \file bmc_dev_fru.h
 *
 * Description: BMC Field Replaceable Unit Inventory Device
 */

#include <malloc.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <pp/base.h>
#include <pp/bmc/ipmi_err.h>
#include <pp/bmc/ipmi_cmd.h>
#include <pp/bmc/ipmi_sess.h>
#include <pp/bmc/ipmi_fru.h>
#include <pp/bmc/bmc_imsg.h>
#include <pp/bmc/bmc_core.h>
#include <pp/bmc/bmc_router.h>
#include <pp/bmc/bmc_config.h>
#include <pp/bmc/debug.h>
#include <pp/bmc/utils.h>
#include <pp/bmc/bmc_fru.h>
#include "bmc_dev_fru.h"

#include <pp/i2c.h>
#include "bmc_dev_oem_asus.h"

#ifdef PRODUCT_SMIDC
  static const char FRU_FILE_MAGIC[] = "SMC-BMC-FRU"; // changed by Edward Pan
  #define FRU_FILE_VER   1
  #define MAX_FRU_SIZE 0x800
  #define IPMI_ERROR_FRU_PROTECTED_AREA	0x80
#else
  static const char FRU_FILE_MAGIC[] = "PPBMC-FRU"; // changed by Edward Pan
  #define FRU_FILE_VER   1
  #define MAX_FRU_SIZE 0x1fff
#endif  

#ifdef PRODUCT_SMIDC
// added by Edward Pan
char fru0_access_lock_flag = 1;  // added by Edward
#endif

#define ASUS_FRU_SIZE 			0x100

static int pp_bmc_fru_read_data_asus(void* buf,  off_t offset, size_t cnt, unsigned char dev_id);

asus_fru_info_t asus_fru[ASUS_MAX_FRU_DEVICE] = {
//	PCA9545_switch_value,	Slave_address(7 bit),	Flags(two_byte_offset, cached, reserved)
	{0,						(0x00 >> 1),			0,0,0},		//FRU_Device_ID(0):BMC host FRU
	{4,						(0x00 >> 1),			0,0,0},		//FRU_Device_ID(1):Backplane 1 FRU
	{4,						(0x00 >> 1),			0,0,0},		//FRU_Device_ID(2):Riser FRU
	{8,						(0x00 >> 1),			0,0,0},		//FRU_Device_ID(3):Power Panel FRU
	{8,						(0x00 >> 1),			0,0,0},		//FRU_Device_ID(4):Power 1 FRU
	{8,						(0x00 >> 1),			0,0,0},		//FRU_Device_ID(5):Power 2 FRU
	{4,						(0x00 >> 1),			0,0,0},		//FRU_Device_ID(6):Backplane 2 FRU
	{4,						(0x00 >> 1),			0,0,0},		//FRU_Device_ID(7):Backplane 3 FRU
	{4,						(0x00 >> 1),			0,0,0},		//FRU_Device_ID(8):Backplane 4 FRU
	{4,						(0x00 >> 1),			0,0,0},		//FRU_Device_ID(9):Fan Module FRU
	{0,						(0x00 >> 1),			0,0,0},		//FRU_Device_ID(10):MotherBoard FRU
	{0,						(0x00 >> 1),			0,0,0},		//FRU_Device_ID(11):Reserved FRU
	{0,						(0x00 >> 1),			0,0,0},		//FRU_Device_ID(12):Reserved FRU
	{0,						(0x00 >> 1),			0,0,0},		//FRU_Device_ID(13):Reserved FRU
	{0,						(0x00 >> 1),			0,0,0},		//FRU_Device_ID(14):Reserved FRU
	{0,						(0x00 >> 1),			0,0,0},		//FRU_Device_ID(15):Reserved FRU
	{0,						(0x00 >> 1),			0,0,0},		//FRU_Device_ID(16):Reserved FRU
	{0,						(0x00 >> 1),			0,0,0},		//FRU_Device_ID(17):Reserved FRU
	{0,						(0x00 >> 1),			0,0,0},		//FRU_Device_ID(18):Reserved FRU
	{0,						(0x00 >> 1),			0,0,0},		//FRU_Device_ID(19):Reserved FRU
	{0,						(0x00 >> 1),			0,0,0},		//FRU_Device_ID(20):Reserved FRU
	{0,						(0x00 >> 1),			0,0,0},		//FRU_Device_ID(21):Reserved FRU
	{0,						(0x00 >> 1),			0,0,0},		//FRU_Device_ID(22):Reserved FRU
	{0,						(0x00 >> 1),			0,0,0},		//FRU_Device_ID(23):Reserved FRU
};	

struct fru_file_header_s {
    char magic[16];
    int ver;
    int offs; /* offset to data: array of ipmi_fru_header_t */
} __attribute__ ((packed));
typedef struct fru_file_header_s fru_file_header_t;

/* static data */
//static struct {
//    char *file_name;
//} fru = { NULL };
static struct {
    char *file_name[ASUS_MAX_FRU_DEVICE];
} fru;


/*
 * FRU device helpers
 */

static int fru_storage_create(void)
{
    assert(fru.file_name[0]);
    int fd = open(fru.file_name[0], O_WRONLY | O_CREAT, S_RW_RW_RW_);
    if (fd == -1) return PP_ERR; /* file not createable */

    /* write empty file with header */
    fru_file_header_t filehdr;
    memset(filehdr.magic, 0, sizeof(filehdr.magic));
    strcpy(filehdr.magic, FRU_FILE_MAGIC);
    filehdr.ver = FRU_FILE_VER;
    filehdr.offs = 0;
    write(fd, &filehdr, sizeof(filehdr));

    { /* the FRU header */
        ipmi_fru_header_t fru_hdr;
        
#ifdef PRODUCT_SMIDC
        fru_hdr.reserved = 0x0;
        fru_hdr.version = IPMI_FRU_HEADER_VER;
        fru_hdr.offset.internal = 0x00;
        fru_hdr.offset.chassis = 0x00;
        fru_hdr.offset.board = 0x01;    // use different board offset
        fru_hdr.offset.product = 0x08;  // use different product offset
        fru_hdr.offset.multi = 0x00;
        fru_hdr.pad = 0x00;
        fru_hdr.checksum = 0xF6; /* zero checksum */
#else
        fru_hdr.reserved = 0x0;
        fru_hdr.version = IPMI_FRU_HEADER_VER;
        fru_hdr.offset.internal = 0x00;
        fru_hdr.offset.chassis = 0x00;
        fru_hdr.offset.board = 0x00;
        fru_hdr.offset.product = 0x00;
        fru_hdr.offset.multi = 0x00;
        fru_hdr.pad = 0x00;
        fru_hdr.checksum = 0xFE; /* zero checksum */
#endif
        write(fd, &fru_hdr, sizeof(ipmi_fru_header_t));
    }
#if 0 /* LARA_KIMMSI */
    { /* the MSI 9136 FRU data */
        static const unsigned char fru9136[104] =
                            { 0x01, 0x01, 0x02, 0x04, 0x08, 0x00, 0x00, 0xF0,
                              0x01, 0x00, 0x01, 0x04, 0x00, 0x00, 0x03, 0xF7,
                              0x01, 0x02, 0x17, 0xC4, 0x39, 0x31, 0x33, 0x36,
                              0xC0, 0xC0, 0xC1, 0x00, 0x00, 0x00, 0x00, 0x0D,
                              0x01, 0x04, 0x00, 0x00, 0x00, 0x00, 0xC3, 0x4D,
                              0x53, 0x49, 0xC4, 0x39, 0x31, 0x33, 0x36, 0xC0,
                              0xC7, 0x4D, 0x53, 0x2D, 0x39, 0x31, 0x33, 0x36,
                              0xC0, 0xC0, 0xC1, 0x00, 0x00, 0x00, 0x00, 0x50,
                              0x01, 0x05, 0x00, 0xC3, 0x53, 0x50, 0x44, 0xC7,
                              0x4D, 0x53, 0x49, 0x39, 0x31, 0x33, 0x36, 0xC7,
                              0x4D, 0x53, 0x2D, 0x39, 0x31, 0x33, 0x36, 0xC3,
                              0x31, 0x30, 0x30, 0xC0, 0xC0, 0xC0, 0xC0, 0xC1,
                              0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x4F };

        write(fd, &fru9136, sizeof(fru9136));
    }
#endif /* LARA_KIMMSI  */
#ifdef PRODUCT_SMIDC
 // changed by Ed
        static const unsigned char supermicro_fru[120] =
                            { 0x01, 0x07, 0x00, 0x00, 0x00, 0x00, 0xCB, 0x53,
                              0x75, 0x70, 0x65, 0x72, 0x20, 0x4D, 0x69, 0x63,
                              0x72, 0x6F, 0xC7, 0x49, 0x50, 0x4D, 0x49, 0x32,
                              0x2E, 0x30, 0xCA, 0x20, 0x20, 0x20, 0x20, 0x20,
                              0x20, 0x20, 0x20, 0x20, 0x20, 0xCC, 0x41, 0x4F,
                              0x43, 0x2D, 0x49, 0x50, 0x4D, 0x49, 0x32, 0x30,
                              0x2D, 0x45, 0xC0, 0xC1, 0x00, 0x00, 0x00, 0x24,
                              0x01, 0x07, 0x00, 0xCB, 0x53, 0x75, 0x70, 0x65,
                              0x72, 0x20, 0x4D, 0x69, 0x63, 0x72, 0x6F, 0xC7,
                              0x49, 0x50, 0x4D, 0x49, 0x32, 0x2E, 0x30, 0xCC,
                              0x41, 0x4F, 0x43, 0x2D, 0x49, 0x50, 0x4D, 0x49,
                              0x32, 0x30, 0x2D, 0x45, 0xC3, 0x31, 0x2E, 0x30,
                              0xCA, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
                              0x20, 0x20, 0x20, 0xC0, 0xC0, 0xC1, 0x00, 0x12,
                              0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};

        write(fd, &supermicro_fru, sizeof(supermicro_fru));
  // end of changed
#endif

    close(fd);

    return PP_SUC;
}

static int fru_storage_check(void)
{
    assert(fru.file_name[0]);
    int fd = open(fru.file_name[0], O_RDONLY);
    if (fd == -1) return PP_ERR; /* not existent or not accessible */

    /* read header */
    fru_file_header_t filehdr;
    ssize_t s = read(fd, &filehdr, sizeof(filehdr));
    close(fd);

    if (s != sizeof(filehdr)
    || 0 != strcmp(filehdr.magic, FRU_FILE_MAGIC)
    || filehdr.ver != FRU_FILE_VER)
    {
        /* bad version or corrupted */
        /* (conversion/repairing goes here in future versions) */
        return PP_ERR;
    }
    return PP_SUC;
}

/********************************************************************
 * FRU command handlers
 */

/*
 * Get FRU Inventory Area Info
 */

struct get_fru_inv_area_info_rs_s {
    unsigned short size_le16;
    BITFIELD2(unsigned char, words : 1, reserved : 7);
} __attribute__ ((packed));
typedef struct get_fru_inv_area_info_rs_s get_fru_inv_area_info_rs_t;

static int fru_cmd_get_fru_inv_area_info(imsg_t *imsg)
{
unsigned char device_id;

	device_id = imsg->data[0];
    if (device_id >= ASUS_MAX_FRU_DEVICE)
    {
        pp_bmc_log_error("[FRU] bad device id %d", device_id);
        return pp_bmc_router_resp_err(imsg, IPMI_ERR_INVALID_DATA_FIELD);
    }
    
    get_fru_inv_area_info_rs_t *rs =
        pp_bmc_imsg_resp(imsg, IPMI_ERR_SUCCESS, sizeof(get_fru_inv_area_info_rs_t));

	if(device_id == 0) {
    rs->size_le16 = cpu_to_le16(MAX_FRU_SIZE);
	} else {
	    rs->size_le16 = ASUS_FRU_SIZE;
	};	
    rs->words = 0;

    return pp_bmc_router_send_msg(imsg);
}

/*
 * Read FRU Data
 */

struct read_fru_data_rq_s {
    unsigned char dev_id;
    unsigned short offs_le16;
    unsigned char cnt;
} __attribute__ ((packed));
typedef struct read_fru_data_rq_s read_fru_data_rq_t;

struct read_fru_data_rs_s {
    unsigned char cnt;
    unsigned char data[0];
} __attribute__ ((packed));
typedef struct read_fru_data_rs_s read_fru_data_rs_t;

static int fru_cmd_read_fru_data(imsg_t *imsg)
{
    read_fru_data_rq_t *rq = (void*)imsg->data;
    unsigned char devid = rq->dev_id;
    int offs = le16_to_cpu(rq->offs_le16);
    int cnt = rq->cnt;
    
    int err = 0, idx, fd, n;
    int i2c;
	unsigned char fru_data[ASUS_FRU_SIZE];
	static unsigned char asus_fru_generate_flag = 0;
	asus_fru_entry_t asus_fru_modify;
	unsigned int fru_add_idx;

    /* check FRU device id: only 0 supported */
    if (devid != 0) {
//	SDR FRU Recorder generate checking
		if (asus_fru_generate_flag == 0) {
			++asus_fru_generate_flag;		// Generate once only
//	Generate ASUS FRU table from file ASUS_FRU_FILE_PATH
		    /* open file */
		    assert(ASUS_FRU_FILE_PATH);
		    if (0 > (fd = open(ASUS_FRU_FILE_PATH, O_RDONLY))) {
		        pp_bmc_log_perror("[FRU-ASUS] can't open file %s:", ASUS_FRU_FILE_PATH);
		        goto fru_table_generate_done;
		    };
		    for (fru_add_idx = 0; fru_add_idx < ASUS_MAX_FRU_DEVICE; fru_add_idx++) {
			    memset(&asus_fru_modify, 0, sizeof(asus_fru_modify));
			    /* read file */
			    n = read(fd, &asus_fru_modify, sizeof(asus_fru_modify));
			    if ((n <= 0) || (n != sizeof(asus_fru_modify))) break;
			    /* modify asus_fru table */
			    asus_fru[asus_fru_modify.fru_id] = asus_fru_modify.asus_fru_entry;
		        pp_bmc_log_info("[FRU-ASUS] Entry add to asus_fru, cnt = %d", (fru_add_idx+1));
			    if (asus_fru[asus_fru_modify.fru_id].two_byte_offset == 1) {
			        pp_bmc_log_notice("[FRU-ASUS] FRU device with two_byte_offset");
			    };
			};
	        close(fd);
		};

 fru_table_generate_done:
//	ASUS_FRU table entry checking	    
		if (devid >= ASUS_MAX_FRU_DEVICE || asus_fru[devid].slave_add == 0x00) {
	        pp_bmc_log_error("[FRU-ASUS] bad device id %d", devid);
        return pp_bmc_router_resp_err(imsg, IPMI_ERR_INVALID_DATA_FIELD);
	    };
//	ASUS_FRU table cached flag checking
		if (asus_fru[devid].cached == 0) {
//	Try to read 256 bytes FRU content if FRU device exist
			pp_bmc_log_notice("[FRU-ASUS] FRU devid=0x%2.2X, 9545_value=0x%2.2X, slave_add=0x%2.2X exist checking",
				devid, asus_fru[devid].pca9545_sw_value, asus_fru[devid].slave_add);
		    i2c = pp_i2c_open_bus(I2C_2_NUMBER, &err);
		    if (err != PP_I2C_NO_ERROR) {
				return pp_bmc_router_resp_err(imsg, IPMI_ERR_ASUS_CMD_SPECIFIC_CODES_I2C_ACCESS_FAIL);
		    };
			pp_bmc_log_notice("[FRU-ASUS] I2C R/W on '%s'", pp_i2c_dev_by_hndl(i2c));
		    pp_i2c_lock(i2c);
		    	//SMBus switch
	        if(asus_fru[devid].pca9545_sw_value != 0) {
	        	pp_i2c_tx_byte_core(i2c, ENABLE_DO_LOG_ERR, PCA9545_SLAVE_ADDR, asus_fru[devid].pca9545_sw_value, &err);
		        if (err != PP_I2C_NO_ERROR) {
				    pp_i2c_unlock(i2c);
				    pp_i2c_close(i2c);
					return pp_bmc_router_resp_err(imsg, IPMI_ERR_MWR_NAK);
	        	};
	        };
	        	//FRU device exist check
			pp_i2c_tx_burst_core(i2c, asus_fru[devid].slave_add, fru_data, 0, &err);
	        if (err != PP_I2C_NO_ERROR) {
			    pp_i2c_unlock(i2c);
			    pp_i2c_close(i2c);
				return pp_bmc_router_resp_err(imsg, IPMI_ERR_MWR_NAK);
	        };
	        	//0x00 offset
	        if(asus_fru[devid].two_byte_offset == 1) {
				pp_i2c_tx_byte_data_core(i2c, ENABLE_DO_LOG_ERR, asus_fru[devid].slave_add, 0x00, 0x00, &err);
		        if (err != PP_I2C_NO_ERROR) {
				    pp_i2c_unlock(i2c);
				    pp_i2c_close(i2c);
					return pp_bmc_router_resp_err(imsg, IPMI_ERR_MWR_NAK);
	        	};
			} else {
				pp_i2c_tx_byte_core(i2c, ENABLE_DO_LOG_ERR, asus_fru[devid].slave_add, 0x00, &err);
		        if (err != PP_I2C_NO_ERROR) {
				    pp_i2c_unlock(i2c);
				    pp_i2c_close(i2c);
					return pp_bmc_router_resp_err(imsg, IPMI_ERR_MWR_NAK);
	        	};
	        };
	        	//Read 256 byte FRU data
	        pp_i2c_rx_burst_core(i2c, asus_fru[devid].slave_add, fru_data, ASUS_FRU_SIZE, &err);
	        if (err != PP_I2C_NO_ERROR) {
			    pp_i2c_unlock(i2c);
			    pp_i2c_close(i2c);
				return pp_bmc_router_resp_err(imsg, IPMI_ERR_MWR_NAK);
	        };
		    pp_i2c_unlock(i2c);
		    pp_i2c_close(i2c);
//Write FRU data to FRU file
		    assert(fru.file_name[devid]);
		    /* open file */
		    if (0 > (fd = open(fru.file_name[devid], O_WRONLY | O_CREAT, S_RW_RW_RW_))) {
		        pp_bmc_log_perror("[FRU-ASUS] can't open storage file %s:", fru.file_name[devid]);
				return pp_bmc_router_resp_err(imsg, IPMI_ERR_ASUS_CMD_SPECIFIC_CODES_FRU_FILE_ACCESS_FAIL);
		    };
		    /* write file */
		    n = write(fd, fru_data, sizeof(fru_data));
		    if (n < 0) {
		        pp_bmc_log_error("[FRU-ASUS] write FRU file error (%u bytes requested from %s)", cnt, fru.file_name[devid]);
		        close(fd);
				return pp_bmc_router_resp_err(imsg, IPMI_ERR_ASUS_CMD_SPECIFIC_CODES_FRU_FILE_ACCESS_FAIL);
		    };
		    pp_bmc_log_notice("[FRU-ASUS] %d bytes read from FRU device and written to %s", n, fru.file_name[devid]);
		    close(fd);
		};
    };
    
    read_fru_data_rs_t *rs = (void*)pp_bmc_imsg_resp(imsg, IPMI_ERR_SUCCESS,
        sizeof(read_fru_data_rs_t) + cnt);

    rs->cnt = cnt;
    // init all vals with 0, in case client wants to read more than currently available
    bzero(rs->data, cnt); 

	if (devid != 0) {
			if (asus_fru[devid].cached == 0) {
//Copy request FRU data size to response msg
				for(idx = 0; idx < cnt; idx++) {
					rs->data[idx] = fru_data[offs + idx];
				};
//Change cached flag to 1
				asus_fru[devid].cached = 1;
		        pp_bmc_log_debug("[FRU-ASUS] read %d bytes with offset %d at first FRU reading", cnt, offs);
			} else {
			    if (pp_bmc_fru_read_data_asus(rs->data, offs, cnt, devid) == PP_ERR) {
			        pp_bmc_log_error("[FRU-ASUS] error reading FRU (devid=%d offs=%d cnt=%d)", devid, offs, cnt);
			        pp_bmc_imsg_err(imsg, IPMI_ERR_UNSPECIFIED);
			    };
			};
	} else {
    if (pp_bmc_fru_read_data(rs->data, offs, cnt) == PP_ERR) {
	        pp_bmc_log_error("[FRU] error reading FRU (devid=%d offs=%d cnt=%d)", devid, offs, cnt);
        pp_bmc_imsg_err(imsg, IPMI_ERR_UNSPECIFIED);
	    };
	};

    return pp_bmc_router_send_msg(imsg);
}

/**
 * read a piece of fru data of given length into given buffer
 * and return the number of actually read bytes for ASUS.
 * return -1 (PP_ERR) if error.
 */
static int pp_bmc_fru_read_data_asus(void* buf,  off_t offset, size_t cnt, unsigned char dev_id) {
    int n, fd;
	
    assert(fru.file_name[dev_id]);
    
    /* open file */
    if (0 > (fd = open(fru.file_name[dev_id], O_RDONLY))) {
        pp_bmc_log_perror("[FRU-ASUS] can't open storage file %s:", fru.file_name[dev_id]);
		return PP_ERR;
    };
//
	pp_bmc_log_debug("[FRU-ASUS] open storage file %s", fru.file_name[dev_id]);
    
    if(lseek(fd, offset, SEEK_CUR) == (off_t)-1) {
        pp_bmc_log_error("[FRU-ASUS] seeking offset %d bytes failed", offset);
    } else {
//      pp_bmc_log_notice("[FRU-ASUS] seeked off_set %d bytes on file %s", offset, fru.file_name[dev_id]);
      pp_bmc_log_debug("[FRU-ASUS] seeked offset %d bytes", offset);
    };
    
    /* read file */
    n = read(fd, buf, cnt);
    if (n < 0) {
        pp_bmc_log_error("[FRU-ASUS] read error (%u bytes requested from %s)", cnt, fru.file_name[dev_id]);
        close(fd);
        return PP_ERR;
    };

    pp_bmc_log_debug("[FRU-ASUS] %d bytes read", n);
    close(fd);
    return n;
}

/**
 * read a piece of fru data of given length into given buffer
 * and return the number of actually read bytes.
 * return -1 (PP_ERR) if error.
 */
int pp_bmc_fru_read_data(void* buf,  off_t offset, size_t cnt) {
    int n, fd;
    fru_file_header_t filehdr;
    
    assert(fru.file_name[0]);
    
    /* open file */
    if (0 > (fd = open(fru.file_name[0], O_RDONLY))) {
        pp_bmc_log_perror("[FRU] can't open storage file %s:", fru.file_name[0]);
	return PP_ERR;
    }
    
    /* read header */
    if (read(fd, &filehdr, sizeof(filehdr)) != sizeof(filehdr)) {
        close(fd);
        pp_bmc_log_error("[FRU] corrupted storage file %s:", fru.file_name[0]);
        return PP_ERR;
    } else {
        pp_bmc_log_debug("[FRU] %d bytes read from %s (filehdr)",
                        sizeof(filehdr), fru.file_name[0]);
    }
    
    if(lseek(fd, filehdr.offs + offset, SEEK_CUR) == (off_t)-1) {
        pp_bmc_log_error("[FRU] seeking %d + %d bytes failed for file %s:",
                         (int)filehdr.offs, offset, fru.file_name[0]);
    } else {
        pp_bmc_log_debug("[FRU] seeked %d + %d bytes in %s", 
                        (int)filehdr.offs, offset, fru.file_name[0]);
    }
    
    /* read file */
    n = read(fd, buf, cnt);
    if (n < 0) {
        pp_bmc_log_error("[FRU] read error (%u bytes requested from %s)", cnt, fru.file_name[0]);
        close(fd);
        return PP_ERR;
    }

    pp_bmc_log_debug("[FRU]: %d bytes read", n);
    close(fd);
    return n;
}

/*
 * Write FRU Data
 */

struct write_fru_data_rq_s {
    unsigned char dev_id;
    unsigned short offs_le16;
    unsigned char data[0];
} __attribute__ ((packed));
typedef struct write_fru_data_rq_s write_fru_data_rq_t;

static int fru_cmd_write_fru_data(imsg_t *imsg)
{
    assert(fru.file_name[0]);

#ifdef PRODUCT_SMIDC
    if(fru0_access_lock_flag == 1) {
        pp_bmc_log_error("[FRU] fru writing area is protected");  
        return pp_bmc_router_resp_err(imsg, IPMI_ERROR_FRU_PROTECTED_AREA);
    }
#endif
    
    /* check FRU device id: only 0 supported */
    if (imsg->data[0] != 0)
    {
        pp_bmc_log_error("[FRU] bad device id %d", imsg->data[0]);
        return pp_bmc_router_resp_err(imsg, IPMI_ERR_INVALID_DATA_FIELD);
    }

    write_fru_data_rq_t *rq = (void*)imsg->data;
    int offs = le16_to_cpu(rq->offs_le16);
    int cnt = imsg->data_size - sizeof(write_fru_data_rq_t);

    /* open file */
    int fd = open(fru.file_name[0], O_RDWR);
    if (fd == -1) {
        pp_bmc_log_perror("[FRU] can't open storage file %s", fru.file_name[0]);
        return pp_bmc_router_resp_err(imsg, IPMI_ERR_UNSPECIFIED);
    }

    /* read header */
    fru_file_header_t filehdr;
    if (read(fd, &filehdr, sizeof(filehdr)) != sizeof(filehdr)) {
        close(fd);
        pp_bmc_log_error("[FRU] corrupted storage file %s", fru.file_name[0]);
        return pp_bmc_router_resp_err(imsg, IPMI_ERR_UNSPECIFIED);
    }
            
    lseek(fd, filehdr.offs + offs, SEEK_CUR);

    /* write file */
    unsigned char size = write(fd, rq->data, cnt);
					 
    if (size != cnt) imsg->data[0] = IPMI_ERR_UNSPECIFIED;
    close(fd);
    
    return pp_bmc_router_resp_msg(imsg, IPMI_ERR_SUCCESS, &size, sizeof(size));
}

/********************************************************************
 * FRU device command table
 */

static const dev_cmd_entry_t fru_cmd_tab[] = {
    {
        .cmd_hndlr = fru_cmd_get_fru_inv_area_info,
        .netfn = IPMI_NETFN_STORAGE, .cmd = IPMI_CMD_GET_FRU_INV_AREA_INFO,
        .min_data_size = 1, .min_priv_level = IPMI_PRIV_USER
    },
    {
        .cmd_hndlr = fru_cmd_read_fru_data,
        .netfn = IPMI_NETFN_STORAGE, .cmd = IPMI_CMD_READ_FRU_DATA,
        .min_data_size = sizeof(read_fru_data_rq_t), .min_priv_level = IPMI_PRIV_USER
    },
    {
        .cmd_hndlr = fru_cmd_write_fru_data,
        .netfn = IPMI_NETFN_STORAGE, .cmd = IPMI_CMD_WRITE_FRU_DATA,
        .min_data_size = sizeof(write_fru_data_rq_t), .min_priv_level = IPMI_PRIV_OPERATOR
    },
    { .cmd_hndlr = NULL }
};

/********************************************************************
 * FRU device c'tor/d'tor
 */

int pp_bmc_dev_fru_init()
{
unsigned int i;
unsigned char dev_id[8];

    /* build file name of persistance storage */
	for(i = 0; i<ASUS_MAX_FRU_DEVICE ;i++) {
	    fru.file_name[i] = malloc(strlen(PP_BMC_FLASH_ROOT) + sizeof("/fruXX"));
	    sprintf(dev_id, "%2.2X", i);
	    strcat(strcat(strcpy(fru.file_name[i], PP_BMC_FLASH_ROOT), "/fru"), dev_id);
	};

    /* check existance and version of persistan storage, and try to create it */
    if (PP_ERR == fru_storage_check()) {
        if (errno != ENOENT) {
            pp_bmc_log_error("[FRU] corrupted storage file %s: %s", fru.file_name, pp_error_string(errno));
            return PP_ERR;
        }
        if (PP_ERR == fru_storage_create()) {
            pp_bmc_log_error("[FRU] can't create storage file %s: %s", fru.file_name, pp_error_string(errno));
            return PP_ERR;
        }
    }

    /* register all entries of cmd tab */
    if (PP_ERR == pp_bmc_core_reg_cmd_tab(fru_cmd_tab)) return PP_ERR;
   
    pp_bmc_log_info("[FRU] device started");

//extern pp_rb_tree_t* sensor_map;
//	pp_bmc_log_info("[FRU test] SDR count = %d", pp_rb_tree_count(sensor_map));


    return PP_SUC;
}

void pp_bmc_dev_fru_cleanup()
{
unsigned int i;

    /* unregister all entries of cmd tab */
    pp_bmc_core_unreg_cmd_tab(fru_cmd_tab);

    free(fru.file_name);
	for(i = 0; i<ASUS_MAX_FRU_DEVICE ;i++) {
	    fru.file_name[i] = NULL;
	};
    
    pp_bmc_log_info("[FRU] device shut down");
}
