/*
 * <3BSD notice header>
 *
 * Copyright (C) 2003-2008 Raritan, Inc. <http://www.raritan.com/>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 * 
 *     * Neither the name of [original copyright holder] nor the names of
 *       its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written
 *       permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * \file bmc_dev_fru.h
 *
 * Description: BMC Field Replaceable Unit Inventory Device
 */

#ifndef __BMC_DEV_FRU_H__
#define __BMC_DEV_FRU_H__

/**
 * Initializes the BMC FRU Inventory device and registers it at the BMC core.
 *
 * @return PP_SUC (0) if success
 *         PP_ERR (-1) if error
 */
int pp_bmc_dev_fru_init(void);

/**
 * Finishes the BMC FRU Inventory device and unregisters it at the BMC core.
 */
void pp_bmc_dev_fru_cleanup(void);

struct asus_fru_info_s {
	unsigned char	pca9545_sw_value;
	unsigned char	slave_add;
    BITFIELD3(unsigned char, two_byte_offset : 1, cached : 1, reserved : 6);	//2-byte offset for 24c32,64,128,256 devices
} __attribute__ ((packed));
typedef struct asus_fru_info_s asus_fru_info_t;

struct asus_fru_entry_s {
	unsigned char fru_id;
	asus_fru_info_t asus_fru_entry;
} __attribute__ ((packed));
typedef struct asus_fru_entry_s asus_fru_entry_t;

#define ASUS_MAX_FRU_DEVICE 		24
#define ASUS_FRU_FILE_PATH			"/tmp/fru_table.cfg"

#endif /* !__BMC_DEV_FRU_H__ */
