/**
 * bmc_dev_oem_asus.c
 *
 * Description: ASUS OEM commands
 *
 */
#include <fcntl.h>

#include <pp/base.h>
#include <pp/cfg.h>

#include <pp/bmc/ipmi_cmd.h>
#include <pp/bmc/ipmi_err.h>
#include <pp/bmc/ipmi_msg.h>
#include <pp/bmc/ipmi_sess.h>
#include <pp/bmc/bmc_imsg.h>
#include <pp/bmc/bmc_core.h>
#include <pp/bmc/bmc_router.h>
#include <pp/bmc/debug.h>
#include <pp/bmc/host_sensors.h>
#include <pp/km.h>
#include <pp/vsc.h>


#include <pp/i2c.h>

#include <pp/bmc/host_i2c_access.h>

#include <pp/rb_tree.h>

#include <pp/bmc/host_board_id.h>


#include "bmc_dev_sensor.h"
#include "bmc_dev_oem_asus.h"


extern pp_rb_tree_t* sensor_map;
extern unsigned char asus_sensor_number_list[128];

asus_write_sensor_info_t asus_write_sensor_table[ASUS_WRITE_SENSOR_TABLE_MAX_NUMBER] = {
//	Sensor_number,	PCA9545_sw_value,	Slave_add(7 bit),	Cmd_offset_byte[0],	Cmd_offset_byte[1],	write_value_mask,	Flags(sdr_check, protocol_type, reserved)
//	{0xB0,			4,					(0x44 >> 1),		{0,					0},					0xFF,				1,0,0},		//FAN module control/LED
//	{0xB0,			4,					(0x46 >> 1),		{0,					0},					0xFF,				1,0,0},		//FAN module control/LED, BP4LSA-AS25 TEST OK
//	{0x55,			4,					(0x5A >> 1),		{0x5C,				0},					0x10,				0,1,0},		//Chassis Fan1 LED
//	{0x55,			4,					(0x58 >> 1),		{0x5C,				0},					0x10,				0,1,0},		//Chassis Fan1 LED, BP4LSA-AS25 TEST OK
//	{0x56,			4,					(0x5C >> 1),		{0x5C,				0},					0x10,				0,1,0},		//Chassis Fan2 LED
//	{0x56,			4,					(0x58 >> 1),		{0x5C,				0},					0x10,				0,1,0},		//Chassis Fan2 LED, BP4LSA-AS25 TEST OK
};

static unsigned char total_mdl_str_number = 0, mbsupmdl_fileread_flag = 0;
static asus_mb_sup_mdl_string_t asus_mb_sup_mdl_table[ASUS_MB_SUPPORT_MODEL_MAX_NUMBER];

static unsigned char asus_bmc_post_init_flag = 0;
static unsigned char asus_eeprom_buf[64];

/*
 * ---------------------------------------------------------------------------
 * Net Function : IPMI_NETFN_OEM_ASUS								0x30
 * Cmd Code     : IPMI_CMD_ASUS_READ_WRITE_SENSOR_VALUE				0x01
 * ---------------------------------------------------------------------------
 */
struct oem_asus_cmd_3001_rq_s {
    unsigned char sensor_number;	/* 0x53 message LED; 0x54 locate LED */
    unsigned char sensor_value;  	/* 0:OFF LED; 1,2,3:ON LED */
} __attribute__ ((packed));
typedef struct oem_asus_cmd_3001_rq_s oem_asus_cmd_3001_rq_t;

struct oem_asus_cmd_3001_rs_s {
    unsigned char sensor_number;
	unsigned char sensor_value;
} __attribute__ ((packed));
typedef struct oem_asus_cmd_3001_rs_s oem_asus_cmd_3001_rs_t;


#ifdef ASUS_GPIO_LED
static int mled_on, lled_on;
static int asus_gpio_mled_read(void)
{
   return(mled_on);
}

static int asus_gpio_lled_read(void)
{
   return(lled_on);
}

static int asus_gpio_mled_write(int sen_value)
{
    pp_tp_gpio_act_t *act = pp_bmc_host_sensor_get_gpio_actor(PP_ACTOR_MLED_EN);
    if (act) {
		act->set_gpio(act, sen_value);
        pp_bmc_log_warn("[HW] MLED gpio signal control success");
	mled_on = sen_value;
	    return PP_SUC;
    } else {
        pp_bmc_log_fatal("[HW] MLED gpio signal control failed");
	    return PP_ERR;    	
    };
}

static int asus_gpio_lled_write(int sen_value)
{
    long i;
    pp_tp_gpio_act_t *act = pp_bmc_host_sensor_get_gpio_actor(PP_ACTOR_LLED_EN);
    if (act) {

	if ( sen_value != lled_on )
	{
		act->set_gpio(act, 0);
		for(i=0; i<1024*1024; i++) ;

		act->set_gpio(act, 1);
		for(i=0; i<1024*1024; i++) ;

		act->set_gpio(act, 0);
	}
        pp_bmc_log_warn("[HW] LLED gpio signal control success");
	lled_on = sen_value;
	    return PP_SUC;
    } else {
        pp_bmc_log_fatal("[HW] LLED gpio signal control failed");
	    return PP_ERR;    	
    };
}

#endif

#ifdef ASUS_w83601_LED
static int asus_w83601_mled_read(imsg_t* imsg)
{
    unsigned char sen_value;

    int err = 0;
    int i2c;
    u_int8_t smb_data_8;


				    pp_bmc_log_info("[OEM-ASUS_CMD_3001_00_53] Get message LED control");
				    i2c = pp_i2c_open_bus(I2C_2_NUMBER, &err);
				    if (err != PP_I2C_NO_ERROR) {
						return pp_bmc_router_resp_err(imsg, IPMI_ERR_INVALID_DATA_FIELD);
				    };
					pp_bmc_log_notice("[OEM-ASUS_CMD] I2C R/W on '%s'", pp_i2c_dev_by_hndl(i2c));
				    pp_i2c_lock(i2c);
		
			        pp_i2c_tx_byte_core(i2c, ENABLE_DO_LOG_ERR, PCA9545_SLAVE_ADDR, 0x02, &err);	//SMBus#5
			        if (err != PP_I2C_NO_ERROR) {
					    pp_i2c_unlock(i2c);
					    pp_i2c_close(i2c);
						return pp_bmc_router_resp_err(imsg, IPMI_ERR_MWR_NAK);
			        };
		
			        smb_data_8 = pp_i2c_rx_byte_data_core(i2c, ENABLE_DO_LOG_ERR, W83601R_SLAVE_ADDR, 0x01, &err);
			        if (err != PP_I2C_NO_ERROR) {
					    pp_i2c_unlock(i2c);
					    pp_i2c_close(i2c);
						return pp_bmc_router_resp_err(imsg, IPMI_ERR_MWR_NAK);
			        };								//Output port data : (GP10 ouput value)
	
				    pp_i2c_unlock(i2c);
				    pp_i2c_close(i2c);

				    sen_value = ((smb_data_8 & 0x01) == 0x01) ? 0x00 : 0x01;	//0x00: LED On, return 0x01  (0x01~0x03)
				return sen_value;
}

static int asus_w83601_mled_write(imsg_t* imsg, int sen_value)
{

    int err = 0;
    int i2c;
    u_int8_t smb_data_8;

			if(sen_value > 3) {
				if (asus_bmc_post_init_flag == 0) {
					return pp_bmc_router_resp_err(imsg, IPMI_ERR_INVALID_DATA_FIELD);
				} else {
					return PP_ERR;
				};
			};
		    pp_bmc_log_info("[OEM-ASUS_CMD_3001_53_%2.2X] Set message LED control", sen_value);

		    i2c = pp_i2c_open_bus(I2C_2_NUMBER, &err);
		    if (err != PP_I2C_NO_ERROR) {
				if (asus_bmc_post_init_flag == 0) {
					return pp_bmc_router_resp_err(imsg, IPMI_ERR_INVALID_DATA_FIELD);
				} else {
					return PP_ERR;
				};
		    };
			pp_bmc_log_notice("[OEM-ASUS_CMD] I2C R/W on '%s'", pp_i2c_dev_by_hndl(i2c));
		    pp_i2c_lock(i2c);

	        pp_i2c_tx_byte_core(i2c, ENABLE_DO_LOG_ERR, PCA9545_SLAVE_ADDR, 0x02, &err);	//SMBus#5
	        if (err != PP_I2C_NO_ERROR) {
			    pp_i2c_unlock(i2c);
			    pp_i2c_close(i2c);
				if (asus_bmc_post_init_flag == 0) {
					return pp_bmc_router_resp_err(imsg, IPMI_ERR_MWR_NAK);
				} else {
					return PP_ERR;
				};
	        };

	        smb_data_8 = pp_i2c_rx_byte_data_core(i2c, ENABLE_DO_LOG_ERR, W83601R_SLAVE_ADDR, 0x02, &err);
	        if (err != PP_I2C_NO_ERROR) {
			    pp_i2c_unlock(i2c);
			    pp_i2c_close(i2c);
				if (asus_bmc_post_init_flag == 0) {
					return pp_bmc_router_resp_err(imsg, IPMI_ERR_MWR_NAK);
				} else {
					return PP_ERR;
				};
	        };
	        smb_data_8 &= (~0x01);			//Polarity : (GP10 NOT Inverted)
			if (asus_bmc_post_init_flag == 1) {
				smb_data_8 &= (~0x04);		//Polarity : (GP12 NOT Inverted)
			};
	        pp_i2c_tx_byte_data_core(i2c, ENABLE_DO_LOG_ERR, W83601R_SLAVE_ADDR, 0x02, smb_data_8, &err);
	        if (err != PP_I2C_NO_ERROR) {
			    pp_i2c_unlock(i2c);
			    pp_i2c_close(i2c);
				if (asus_bmc_post_init_flag == 0) {
					return pp_bmc_router_resp_err(imsg, IPMI_ERR_MWR_NAK);
				} else {
					return PP_ERR;
				};
	        };
		
	        smb_data_8 = pp_i2c_rx_byte_data_core(i2c, ENABLE_DO_LOG_ERR, W83601R_SLAVE_ADDR, 0x04, &err);
	        smb_data_8 &= (~0x01);			//Output style : (GP10 Level)
			if (asus_bmc_post_init_flag == 1) {
				smb_data_8 &= (~0x04);		//Output style : (GP12 Level)
			};
	        pp_i2c_tx_byte_data_core(i2c, ENABLE_DO_LOG_ERR, W83601R_SLAVE_ADDR, 0x04, smb_data_8, &err);

	        smb_data_8 = pp_i2c_rx_byte_data_core(i2c, ENABLE_DO_LOG_ERR, W83601R_SLAVE_ADDR, 0x01, &err);
			if (sen_value == 0) {			//Output port data : (GP10 Output value)
				smb_data_8 |= 0x01;		//Off-LED : 0 (GP10 ouput high)
			} else {
				smb_data_8 &= (~0x01);	//On-LED : 1,2,3 (GP10 ouput low)
			};
			if (asus_bmc_post_init_flag == 1) {
				smb_data_8 &= (~0x04);		//Output port data : (GP12 Output low)
			};
	        pp_i2c_tx_byte_data_core(i2c, ENABLE_DO_LOG_ERR, W83601R_SLAVE_ADDR, 0x01, smb_data_8, &err);

	        smb_data_8 = pp_i2c_rx_byte_data_core(i2c, ENABLE_DO_LOG_ERR, W83601R_SLAVE_ADDR, 0x03, &err);
	        smb_data_8 &= (~0x01);			//Input_Output config : (GP10 Output)
			if (asus_bmc_post_init_flag == 1) {
				smb_data_8 &= (~0x04);		//Input_Output config : (GP12 Output)
			};
	        pp_i2c_tx_byte_data_core(i2c, ENABLE_DO_LOG_ERR, W83601R_SLAVE_ADDR, 0x03, smb_data_8, &err);

		    pp_i2c_unlock(i2c);
		    pp_i2c_close(i2c);

		return 0;
}


static int asus_w83601_lled_read(imsg_t* imsg)
{
    unsigned char sen_value;

    int err = 0;
    int i2c;
	u_int8_t smb_data_8;


				    pp_bmc_log_info("[OEM-ASUS_CMD_3001_00_54] Get locate LED control");
				    i2c = pp_i2c_open_bus(I2C_2_NUMBER, &err);
				    if (err != PP_I2C_NO_ERROR) {
						return pp_bmc_router_resp_err(imsg, IPMI_ERR_INVALID_DATA_FIELD);
				    };
					pp_bmc_log_notice("[OEM-ASUS_CMD] I2C R/W on '%s'", pp_i2c_dev_by_hndl(i2c));
				    pp_i2c_lock(i2c);
		
			        pp_i2c_tx_byte_core(i2c, ENABLE_DO_LOG_ERR, PCA9545_SLAVE_ADDR, 0x02, &err);	//SMBus#5
			        if (err != PP_I2C_NO_ERROR) {
					    pp_i2c_unlock(i2c);
					    pp_i2c_close(i2c);
						return pp_bmc_router_resp_err(imsg, IPMI_ERR_MWR_NAK);
			        };
		
			        smb_data_8 = pp_i2c_rx_byte_data_core(i2c, ENABLE_DO_LOG_ERR, W83601R_SLAVE_ADDR, 0x08, &err);
			        if (err != PP_I2C_NO_ERROR) {
					    pp_i2c_unlock(i2c);
					    pp_i2c_close(i2c);
						return pp_bmc_router_resp_err(imsg, IPMI_ERR_MWR_NAK);
			        };
	
				    pp_i2c_unlock(i2c);
				    pp_i2c_close(i2c);

					sen_value = ((smb_data_8 & 0x20) >> 5) == 0x01 ? 0x00 : 0x01; // 1:OFF(return 0) , 0:ON(return 1) (1,2,3)
				return sen_value;

}

static int asus_w83601_lled_write(imsg_t* imsg, int sen_value)
{
    int err = 0;
    int i2c;
	u_int8_t smb_data_8;

			if(sen_value > 3) {
				if (asus_bmc_post_init_flag == 0) {
					return pp_bmc_router_resp_err(imsg, IPMI_ERR_INVALID_DATA_FIELD);
				} else {
					return PP_ERR;
				};
			};
		    pp_bmc_log_info("[OEM-ASUS_CMD_3001_54_%2.2X] Set locate LED control", sen_value);
		    i2c = pp_i2c_open_bus(I2C_2_NUMBER, &err);
		    if (err != PP_I2C_NO_ERROR) {
				if (asus_bmc_post_init_flag == 0) {
					return pp_bmc_router_resp_err(imsg, IPMI_ERR_INVALID_DATA_FIELD);
				} else {
					return PP_ERR;
				};
		    };
			pp_bmc_log_notice("[OEM-ASUS_CMD] I2C R/W on '%s'", pp_i2c_dev_by_hndl(i2c));
		    pp_i2c_lock(i2c);

	        pp_i2c_tx_byte_core(i2c, ENABLE_DO_LOG_ERR, PCA9545_SLAVE_ADDR, 0x02, &err);	//SMBus#5
	        if (err != PP_I2C_NO_ERROR) {
			    pp_i2c_unlock(i2c);
			    pp_i2c_close(i2c);
				if (asus_bmc_post_init_flag == 0) {
					return pp_bmc_router_resp_err(imsg, IPMI_ERR_MWR_NAK);
				} else {
					return PP_ERR;
				};
	        };

	        smb_data_8 = pp_i2c_rx_byte_data_core(i2c, ENABLE_DO_LOG_ERR, W83601R_SLAVE_ADDR, 0x0A, &err);
	        if (err != PP_I2C_NO_ERROR) {
			    pp_i2c_unlock(i2c);
			    pp_i2c_close(i2c);
				if (asus_bmc_post_init_flag == 0) {
					return pp_bmc_router_resp_err(imsg, IPMI_ERR_MWR_NAK);
				} else {
					return PP_ERR;
				};
	        };
	        smb_data_8 &= (~0x60);			//Polarity : (GP25,26 Not inverted)
	        pp_i2c_tx_byte_data_core(i2c, ENABLE_DO_LOG_ERR, W83601R_SLAVE_ADDR, 0x0A, smb_data_8, &err);
	        if (err != PP_I2C_NO_ERROR) {
			    pp_i2c_unlock(i2c);
			    pp_i2c_close(i2c);
				if (asus_bmc_post_init_flag == 0) {
					return pp_bmc_router_resp_err(imsg, IPMI_ERR_MWR_NAK);
				} else {
					return PP_ERR;
				};
	        };
		
	        smb_data_8 = pp_i2c_rx_byte_data_core(i2c, ENABLE_DO_LOG_ERR, W83601R_SLAVE_ADDR, 0x0C, &err);
	        smb_data_8 |= 0x40;				//Output style : (GP26 Pulse)
	        pp_i2c_tx_byte_data_core(i2c, ENABLE_DO_LOG_ERR, W83601R_SLAVE_ADDR, 0x0C, smb_data_8, &err);

	        smb_data_8 = pp_i2c_rx_byte_data_core(i2c, ENABLE_DO_LOG_ERR, W83601R_SLAVE_ADDR, 0x0B, &err);
	        smb_data_8 |= 0x60;				//Input_Output config : (GP25,26 Input)
	        pp_i2c_tx_byte_data_core(i2c, ENABLE_DO_LOG_ERR, W83601R_SLAVE_ADDR, 0x0B, smb_data_8, &err);

	        smb_data_8 = pp_i2c_rx_byte_data_core(i2c, ENABLE_DO_LOG_ERR, W83601R_SLAVE_ADDR, 0x08, &err);
			smb_data_8 = (smb_data_8 & 0x20) >> 5; // 1:OFF , 0:ON
			if ( (sen_value == 0 && smb_data_8 == 1)  ||
				((sen_value >= 1 && sen_value <= 3) && smb_data_8 == 0) ) {
				    pp_i2c_unlock(i2c);
				    pp_i2c_close(i2c);
					return PP_ERR;
					//break;
			};

	        smb_data_8 = pp_i2c_rx_byte_data_core(i2c, ENABLE_DO_LOG_ERR, W83601R_SLAVE_ADDR, 0x09, &err);
	        smb_data_8 &= (~0x40);			//Toggle LED : (GP26 LOW pulse output)
	        pp_i2c_tx_byte_data_core(i2c, ENABLE_DO_LOG_ERR, W83601R_SLAVE_ADDR, 0x09, smb_data_8, &err);

	        smb_data_8 = pp_i2c_rx_byte_data_core(i2c, ENABLE_DO_LOG_ERR, W83601R_SLAVE_ADDR, 0x0B, &err);
	        smb_data_8 &= (~0x40);			//Input_Output config : GP26 (Output)
	        pp_i2c_tx_byte_data_core(i2c, ENABLE_DO_LOG_ERR, W83601R_SLAVE_ADDR, 0x0B, smb_data_8, &err);

		    pp_i2c_unlock(i2c);
		    pp_i2c_close(i2c);

		return 0;
}

#endif

static int oem_asus_cmd_3001(imsg_t* imsg) {

    oem_asus_cmd_3001_rq_t* rq = (void*)imsg->data;
    oem_asus_cmd_3001_rs_t* rs;

    unsigned char resp_code, sen_number, sen_value;
    ipmi_sensor_get_reading_rs_t resp_data;
    pp_tp_ipmi_sens_t* sensor;

    int err = 0;
    int i2c;
	u_int8_t smb_data_8;
	unsigned int asus_write_sensor_table_index;

	static unsigned char asus_write_sensor_table_generate_flag = 0;
    int fd, n;
	asus_write_sensor_info_t asus_write_sensor_new;
	static unsigned int asus_write_sensor_add_cnt = 0;
    const char *cname = "[Writable_Snr-ASUS]";


	
	if(imsg->data_size > sizeof(oem_asus_cmd_3001_rq_t))
		return pp_bmc_router_resp_err(imsg, IPMI_ERR_REQ_DATA_LEN_LIMIT_EXCEEDED);
//	if(imsg->data_size < 2)
//		return pp_bmc_router_resp_err(imsg, IPMI_ERR_REQ_DATA_TRUNC);
	sen_number = rq->sensor_number;
	sen_value = rq->sensor_value;
	switch (sen_number) {
      case 0x00:		//Read sensor value
      		sen_number = sen_value;
			switch (sen_number) {
		      case 0x53:
				 #ifdef ASUS_w83601_LED
				 sen_value=asus_w83601_mled_read(imsg);
				 #endif

				 #ifdef ASUS_GPIO_LED
				 sen_value = asus_gpio_mled_read();
				 #endif

				  break;
		      case 0x54:
				 #ifdef ASUS_w83601_LED
				 sen_value=asus_w83601_lled_read(imsg);
				 #endif

				 #ifdef ASUS_GPIO_LED
				 sen_value = asus_gpio_lled_read();
				 #endif
				  break;
		      default:	
				    pp_bmc_log_info("[OEM-ASUS_CMD_3001_00_%2.2X] Read sensor value", sen_number);
					sensor = get_sensor_by_index(sen_number);
				    if (!sensor) {
				        return pp_bmc_router_resp_err(imsg, IPMI_ERR_ITEM_NOT_PRESENT);
				    };
				    if (PP_ERR == pp_tp_ipmi_sens_get_reading(sensor, &resp_code, &resp_data)){
						if (errno == ENOSYS) {
						    return pp_bmc_router_resp_err(imsg, IPMI_ERR_CMD_ILLEGAL_FOR_THIS_ITEM);
						} else {
						    pp_bmc_log_error("[SD]: pp_tp_ipmi_sens_get_reading from %s "
								      "failed", pp_tp_ipmi_sens_to_string(sensor));
						    return pp_bmc_router_resp_err(imsg, resp_code);
						};
				    };
				    sen_value = resp_data.rt.value;
				  break;
			};
		  break;
      case 0x53:		//Write message LED value
				asus_w83601_mled_write(imsg, sen_value);
				//asus_gpio_mled_write(sen_value);

		  break;
      case 0x54:		//Write locate LED value
				asus_w83601_lled_write(imsg, sen_value);
				//asus_gpio_lled_write(sen_value);

		  break;
      default:	
//	ASUS writable sensor table generate checking
			if (asus_write_sensor_table_generate_flag == 0) {
				++asus_write_sensor_table_generate_flag;		// Generate once only
//	Generate ASUS writable sensor table from file ASUS_WRITE_SENSOR_TABLE_FILE_PATH
			    /* open file */
			    assert(ASUS_WRITE_SENSOR_TABLE_FILE_PATH);
			    if (0 > (fd = open(ASUS_WRITE_SENSOR_TABLE_FILE_PATH, O_RDONLY))) {
			        pp_bmc_log_error("%s can't open file %s", cname, ASUS_WRITE_SENSOR_TABLE_FILE_PATH);
			        goto writable_sensor_table_generate_done;
			    };
			    for (asus_write_sensor_add_cnt = 0; asus_write_sensor_add_cnt < ASUS_WRITE_SENSOR_TABLE_MAX_NUMBER; asus_write_sensor_add_cnt++) {
				    memset(&asus_write_sensor_new, 0, sizeof(asus_write_sensor_new));
				    /* read file */
				    n = read(fd, &asus_write_sensor_new, sizeof(asus_write_sensor_new));
				    if ((n <= 0) || (n != sizeof(asus_write_sensor_new))) break;
				    /* append new entry to asus_write_sensor_table table */
				    asus_write_sensor_table[asus_write_sensor_add_cnt] = asus_write_sensor_new;
			        pp_bmc_log_info("%s Entry add to asus_write_sensor_table, cnt = %d", cname, (asus_write_sensor_add_cnt+1));
			        pp_bmc_log_notice("%s snr_number = 0x%2.2X, sdr_check = %d, protocol_type = %d",
				        			cname,
				        			asus_write_sensor_new.sensor_number,
				        			asus_write_sensor_new.sdr_check,
				        			asus_write_sensor_new.protocol_type);
//			        pp_bmc_log_notice("%s sw = 0x%2.2X, SA = 0x%2.2X, cmd[0] = 0x%2.2X, cmd[1] = 0x%2.2X, wrmask = 0x%2.2X",
//			        				cname,
//									asus_write_sensor_new.pca9545_sw_value,
//									asus_write_sensor_new.slave_add,
//									asus_write_sensor_new.cmd_offset_byte[0],
//									asus_write_sensor_new.cmd_offset_byte[1],
//									asus_write_sensor_new.write_value_mask);
				};
		        close(fd);
			};

 writable_sensor_table_generate_done:
		    pp_bmc_log_info("[OEM-ASUS_CMD_3001_%2.2X_%2.2X] Write sensor value", rq->sensor_number, rq->sensor_value);
// Check sensor number exist in asus_write_sensor_table
			for(asus_write_sensor_table_index = 0; asus_write_sensor_table_index < asus_write_sensor_add_cnt; asus_write_sensor_table_index++) {
			    if(asus_write_sensor_table[asus_write_sensor_table_index].sensor_number == sen_number)
			    	break;
			};
			if(asus_write_sensor_table_index >= asus_write_sensor_add_cnt) {
		        return pp_bmc_router_resp_err(imsg, IPMI_ERR_ASUS_CMD_SPECIFIC_CODES_OEM_CMD_FAIL);
		    };

// Check sensor number exist in SDRR
		    if(asus_write_sensor_table[asus_write_sensor_table_index].sdr_check == 1) {
				sensor = get_sensor_by_index(sen_number);
			    if (!sensor) {
			        return pp_bmc_router_resp_err(imsg, IPMI_ERR_ASUS_CMD_SPECIFIC_CODES_OEM_CMD_FAIL);
			    };
			};

// Open and lock MB SMBus device
		    i2c = pp_i2c_open_bus(I2C_2_NUMBER, &err);
		    if (err != PP_I2C_NO_ERROR) {
				return pp_bmc_router_resp_err(imsg, IPMI_ERR_INVALID_DATA_FIELD);
		    };
			pp_bmc_log_notice("[OEM-ASUS_CMD] I2C R/W on '%s'", pp_i2c_dev_by_hndl(i2c));
		    pp_i2c_lock(i2c);

// Switch PCA9545
	        pp_i2c_tx_byte_core(i2c, ENABLE_DO_LOG_ERR, PCA9545_SLAVE_ADDR,
	        				asus_write_sensor_table[asus_write_sensor_table_index].pca9545_sw_value, &err);
	        if (err != PP_I2C_NO_ERROR) {
			    pp_i2c_unlock(i2c);
			    pp_i2c_close(i2c);
				return pp_bmc_router_resp_err(imsg, IPMI_ERR_MWR_NAK);
	        };

// Read-modify-write register
	        switch(asus_write_sensor_table[asus_write_sensor_table_index].protocol_type) {
	        	case 0x00:	// 0 : tx_byte		(send byte	<null offset>		: PCF8574, ... etc)
			        smb_data_8 = pp_i2c_rx_byte_core(i2c, ENABLE_DO_LOG_ERR,
			        							asus_write_sensor_table[asus_write_sensor_table_index].slave_add, &err);
			        if (err != PP_I2C_NO_ERROR) {
					    pp_i2c_unlock(i2c);
					    pp_i2c_close(i2c);
						return pp_bmc_router_resp_err(imsg, IPMI_ERR_MWR_NAK);
			        };
			        smb_data_8 &= (~asus_write_sensor_table[asus_write_sensor_table_index].write_value_mask);
			        smb_data_8 |= sen_value & (asus_write_sensor_table[asus_write_sensor_table_index].write_value_mask);
			        pp_i2c_tx_byte_core(i2c, ENABLE_DO_LOG_ERR,
			        				asus_write_sensor_table[asus_write_sensor_table_index].slave_add, smb_data_8, &err);
			        if (err != PP_I2C_NO_ERROR) {
					    pp_i2c_unlock(i2c);
					    pp_i2c_close(i2c);
						return pp_bmc_router_resp_err(imsg, IPMI_ERR_MWR_NAK);
			        };
	        		break;
	        	case 0x01:	// 1 : tx_byte_data	(write byte	<one byte offset>	: ADT7463, ... etc)
			        smb_data_8 = pp_i2c_rx_byte_data_core(i2c, ENABLE_DO_LOG_ERR,
			        					asus_write_sensor_table[asus_write_sensor_table_index].slave_add,
			        					asus_write_sensor_table[asus_write_sensor_table_index].cmd_offset_byte[0], &err);
			        if (err != PP_I2C_NO_ERROR) {
					    pp_i2c_unlock(i2c);
					    pp_i2c_close(i2c);
						return pp_bmc_router_resp_err(imsg, IPMI_ERR_MWR_NAK);
			        };
			        smb_data_8 &= (~asus_write_sensor_table[asus_write_sensor_table_index].write_value_mask);
			        smb_data_8 |= sen_value & (asus_write_sensor_table[asus_write_sensor_table_index].write_value_mask);
			        pp_i2c_tx_byte_data_core(i2c, ENABLE_DO_LOG_ERR,
			        		asus_write_sensor_table[asus_write_sensor_table_index].slave_add,
			        		asus_write_sensor_table[asus_write_sensor_table_index].cmd_offset_byte[0], smb_data_8, &err);
			        if (err != PP_I2C_NO_ERROR) {
					    pp_i2c_unlock(i2c);
					    pp_i2c_close(i2c);
						return pp_bmc_router_resp_err(imsg, IPMI_ERR_MWR_NAK);
			        };
	        		break;
	        	case 0x02:	// 2 : mixed		(<two byte offset>	: AT24C32/64/128/256, ... etc)
			        pp_i2c_tx_byte_data_core(i2c, ENABLE_DO_LOG_ERR,
			        		asus_write_sensor_table[asus_write_sensor_table_index].slave_add,
			        		asus_write_sensor_table[asus_write_sensor_table_index].cmd_offset_byte[0],
			        		asus_write_sensor_table[asus_write_sensor_table_index].cmd_offset_byte[1], &err);
			        if (err != PP_I2C_NO_ERROR) {
					    pp_i2c_unlock(i2c);
					    pp_i2c_close(i2c);
						return pp_bmc_router_resp_err(imsg, IPMI_ERR_MWR_NAK);
			        };
			        smb_data_8 = pp_i2c_rx_byte_core(i2c, ENABLE_DO_LOG_ERR,
			        					asus_write_sensor_table[asus_write_sensor_table_index].slave_add, &err);
			        if (err != PP_I2C_NO_ERROR) {
					    pp_i2c_unlock(i2c);
					    pp_i2c_close(i2c);
						return pp_bmc_router_resp_err(imsg, IPMI_ERR_MWR_NAK);
			        };
			        smb_data_8 &= (~asus_write_sensor_table[asus_write_sensor_table_index].write_value_mask);
			        smb_data_8 |= sen_value & (asus_write_sensor_table[asus_write_sensor_table_index].write_value_mask);
			        pp_i2c_tx_byte_data_core(i2c, ENABLE_DO_LOG_ERR,
			        		asus_write_sensor_table[asus_write_sensor_table_index].slave_add,
			        		asus_write_sensor_table[asus_write_sensor_table_index].cmd_offset_byte[0],
			        		asus_write_sensor_table[asus_write_sensor_table_index].cmd_offset_byte[1], &err);
			        if (err != PP_I2C_NO_ERROR) {
					    pp_i2c_unlock(i2c);
					    pp_i2c_close(i2c);
						return pp_bmc_router_resp_err(imsg, IPMI_ERR_MWR_NAK);
			        };
			        pp_i2c_tx_byte_core(i2c, ENABLE_DO_LOG_ERR,
			        		asus_write_sensor_table[asus_write_sensor_table_index].slave_add, smb_data_8, &err);
			        if (err != PP_I2C_NO_ERROR) {
					    pp_i2c_unlock(i2c);
					    pp_i2c_close(i2c);
						return pp_bmc_router_resp_err(imsg, IPMI_ERR_MWR_NAK);
			        };
	        		break;
	        	case 0x03:	// 3 : rerserved
				    pp_i2c_unlock(i2c);
				    pp_i2c_close(i2c);
					return pp_bmc_router_resp_err(imsg, IPMI_ERR_INVALID_DATA_FIELD);
	        	default:
				    pp_i2c_unlock(i2c);
				    pp_i2c_close(i2c);
					return pp_bmc_router_resp_err(imsg, IPMI_ERR_INVALID_DATA_FIELD);
	        		break;
	        };
		    pp_i2c_unlock(i2c);
		    pp_i2c_close(i2c);
		  break;
   };
	if (asus_bmc_post_init_flag == 0) {
		rs = pp_bmc_imsg_resp(imsg, IPMI_ERR_SUCCESS, sizeof(oem_asus_cmd_3001_rs_t));    
		rs->sensor_number = sen_number;
		rs->sensor_value = sen_value;
		return pp_bmc_router_send_msg(imsg);
	} else {
		return PP_SUC;
	};
}

/*
 * ---------------------------------------------------------------------------
 * Net Function : IPMI_NETFN_OEM_ASUS								0x30
 * Cmd Code     : IPMI_CMD_ASUS_ENABLE_BMC_SCAN_I2C_BUS				0x02
 * ---------------------------------------------------------------------------
 */
struct oem_asus_cmd_3002_rq_s {
    unsigned char value;  	/* 0:Disable BMC scan I2C bus , 1:Enable BMC scan I2C bus */
} __attribute__ ((packed));
typedef struct oem_asus_cmd_3002_rq_s oem_asus_cmd_3002_rq_t;

static int oem_asus_cmd_3002(imsg_t* imsg) {

    oem_asus_cmd_3002_rq_t* rq = (void*)imsg->data;
    unsigned char cnt = 0;
	

	if(imsg->data_size > sizeof(oem_asus_cmd_3002_rq_t))
		return pp_bmc_router_resp_err(imsg, IPMI_ERR_REQ_DATA_LEN_LIMIT_EXCEEDED);
//	if(imsg->data_size < 1)
//		return pp_bmc_router_resp_err(imsg, IPMI_ERR_REQ_DATA_TRUNC);
	switch(rq->value) {
		case 0x00:	//Disable BMC scan I2C bus
			do {
				if ( PP_SUCCED(pp_bmc_sensor_scanner_stop()) ) {
					pp_bmc_log_warn("[OEM-ASUS_CMD] Stop BMC sensor scanner success");
					break;
				};
				pp_bmc_log_error("[OEM-ASUS_CMD] Stop BMC sensor scanner failed, try count #%d", cnt+1);
			} while (++cnt < 5);
			if(cnt >= 5) {
				return pp_bmc_router_resp_err(imsg, IPMI_ERR_ASUS_CMD_SPECIFIC_CODES_OEM_CMD_FAIL);
			};
			if( PP_FAILED(pp_bmc_host_i2c_access_disable()) )
				return pp_bmc_router_resp_err(imsg, IPMI_ERR_ASUS_CMD_SPECIFIC_CODES_OEM_CMD_FAIL);
			if(pp_bmc_host_i2c_access_get_state() != 0x00)
				return pp_bmc_router_resp_err(imsg, IPMI_ERR_ASUS_CMD_SPECIFIC_CODES_OEM_CMD_FAIL);
			break;
		case 0x01:	//Enable BMC scan I2C bus
			if( PP_FAILED(pp_bmc_host_i2c_access_enable()) )
				return pp_bmc_router_resp_err(imsg, IPMI_ERR_ASUS_CMD_SPECIFIC_CODES_OEM_CMD_FAIL);
			if(pp_bmc_host_i2c_access_get_state() != 0x01)
				return pp_bmc_router_resp_err(imsg, IPMI_ERR_ASUS_CMD_SPECIFIC_CODES_OEM_CMD_FAIL);
			do {
				if ( PP_SUCCED(pp_bmc_sensor_scanner_start()) ) {
					pp_bmc_log_warn("[OEM-ASUS_CMD] Start BMC sensor scanner success");
					break;
				};
				pp_bmc_log_error("[OEM-ASUS_CMD] Start BMC sensor scanner failed, try count #%d", cnt+1);
			} while (++cnt < 5);
			if(cnt >= 5) {
				return pp_bmc_router_resp_err(imsg, IPMI_ERR_ASUS_CMD_SPECIFIC_CODES_OEM_CMD_FAIL);
			};
			break;
		default:
			return pp_bmc_router_resp_err(imsg, IPMI_ERR_PARAM_OUT_OF_RANGE);
			break;
	};
	return pp_bmc_router_resp_err(imsg, IPMI_ERR_SUCCESS);
}

/*
 * ---------------------------------------------------------------------------
 * Net Function : IPMI_NETFN_OEM_ASUS								0x30
 * Cmd Code     : IPMI_CMD_ASUS_WRITE_I2C_BYTE						0x03
 * ---------------------------------------------------------------------------
 */

/*
 * ---------------------------------------------------------------------------
 * Net Function : IPMI_NETFN_OEM_ASUS								0x30
 * Cmd Code     : IPMI_CMD_ASUS_GET_SENSOR_DATA_READY				0x04
 * ---------------------------------------------------------------------------
 */

/*
 * ---------------------------------------------------------------------------
 * Net Function : IPMI_NETFN_OEM_ASUS								0x30
 * Cmd Code     : IPMI_CMD_ASUS_GET_SUPPORT_MOTHERBOARD_TYPE		0x05
 * ---------------------------------------------------------------------------
 */

/*
 * ---------------------------------------------------------------------------
 * Net Function : IPMI_NETFN_OEM_ASUS								0x30
 * Cmd Code     : IPMI_CMD_ASUS_READ_ALL_SENSOR_VALUE				0x06
 * ---------------------------------------------------------------------------
 */
struct oem_asus_cmd_3006_rq_s {
    unsigned char record_id;  		/* 0x00: First Record ID */
} __attribute__ ((packed));
typedef struct oem_asus_cmd_3006_rq_s oem_asus_cmd_3006_rq_t;

struct oem_asus_cmd_3006_rs_data_pair_s {
    unsigned char sensor_number;
	unsigned char sensor_value;
} __attribute__ ((packed));
typedef struct oem_asus_cmd_3006_rs_data_pair_s oem_asus_cmd_3006_rs_data_pair_t;

struct oem_asus_cmd_3006_rs_s {
    unsigned char next_record_id;	/* 0xFF: Last Record ID */
	oem_asus_cmd_3006_rs_data_pair_t data[0];
} __attribute__ ((packed));
typedef struct oem_asus_cmd_3006_rs_s oem_asus_cmd_3006_rs_t;

static int oem_asus_cmd_3006(imsg_t* imsg) {

    oem_asus_cmd_3006_rq_t* rq = (void*)imsg->data;
    oem_asus_cmd_3006_rs_t* rs;

    unsigned char resp_code, sen_number;
    ipmi_sensor_get_reading_rs_t resp_data;
    pp_tp_ipmi_sens_t* sensor;

	unsigned char asus_sensor_value_list[128], record_id_backup, next_record_id;
	unsigned int sdr_count, resp_index_begin, resp_index_end, i;
	

	if(imsg->data_size > sizeof(oem_asus_cmd_3006_rq_t))
		return pp_bmc_router_resp_err(imsg, IPMI_ERR_REQ_DATA_LEN_LIMIT_EXCEEDED);
//	if(imsg->data_size < 1)
//		return pp_bmc_router_resp_err(imsg, IPMI_ERR_REQ_DATA_TRUNC);

    pp_bmc_log_info("[OEM-ASUS_CMD_3006_%2.2X] Read All Sensor Value", rq->record_id);
	record_id_backup = rq->record_id;
	sdr_count = pp_rb_tree_count(sensor_map);
    resp_index_begin = record_id_backup * 30;
    if(sdr_count <= resp_index_begin) {
		return pp_bmc_router_resp_err(imsg, IPMI_ERR_PARAM_OUT_OF_RANGE);
	};
    resp_index_end = sdr_count;
	next_record_id = 0xFF;
    if(sdr_count > (resp_index_begin + 30)) {
		next_record_id = resp_index_begin + 1;
        resp_index_end = resp_index_begin + 30;
    };

    for(i = resp_index_begin; i < resp_index_end; i ++) {
        sen_number = asus_sensor_number_list[i];        
		sensor = get_sensor_by_index(sen_number);
	    if (!sensor) {
	        return pp_bmc_router_resp_err(imsg, IPMI_ERR_ASUS_CMD_SPECIFIC_CODES_OEM_CMD_FAIL);
	    };
	    if (PP_ERR == pp_tp_ipmi_sens_get_reading(sensor, &resp_code, &resp_data)){
			if (errno == ENOSYS) {
			    return pp_bmc_router_resp_err(imsg, IPMI_ERR_ASUS_CMD_SPECIFIC_CODES_OEM_CMD_FAIL);
			} else {
			    pp_bmc_log_error("[OEM-ASUS_CMD_3006_%2.2X]: pp_tp_ipmi_sens_get_reading from %s "
					      "failed", record_id_backup, pp_tp_ipmi_sens_to_string(sensor));
			    return pp_bmc_router_resp_err(imsg, IPMI_ERR_ASUS_CMD_SPECIFIC_CODES_OEM_CMD_FAIL);
			};
	    };
	    asus_sensor_value_list[i] = resp_data.rt.value;
	};

	rs = pp_bmc_imsg_resp(imsg, IPMI_ERR_SUCCESS, sizeof(oem_asus_cmd_3006_rs_t)
								+ (resp_index_end - resp_index_begin) * 2);    
	rs->next_record_id = next_record_id;
    for(i = 0; i < (resp_index_end - resp_index_begin); i++) {
		rs->data[i].sensor_number = asus_sensor_number_list[resp_index_begin + i];
		rs->data[i].sensor_value = asus_sensor_value_list[resp_index_begin + i];
	};
	return pp_bmc_router_send_msg(imsg);
}

/*
 * ---------------------------------------------------------------------------
 * Net Function : IPMI_NETFN_OEM_ASUS								0x30
 * Cmd Code     : IPMI_CMD_ASUS_GET_OEM_DEVICE_ID					0x07
 * ---------------------------------------------------------------------------
 */

/*
 * ---------------------------------------------------------------------------
 * Net Function : IPMI_NETFN_OEM_ASUS								0x30
 * Cmd Code     : IPMI_CMD_ASUS_BIOS_POST_CODE						0x08
 * ---------------------------------------------------------------------------
 */
struct oem_asus_cmd_3008_rq_s {
    unsigned char mode;
    unsigned char postcode;
} __attribute__ ((packed));
typedef struct oem_asus_cmd_3008_rq_s oem_asus_cmd_3008_rq_t;

struct oem_asus_cmd_3008_rs_s {
    unsigned char post_code;
} __attribute__ ((packed));
typedef struct oem_asus_cmd_3008_rs_s oem_asus_cmd_3008_rs_t;

static int oem_asus_cmd_3008(imsg_t* imsg) {

    oem_asus_cmd_3008_rq_t* rq = (void*)imsg->data;
    oem_asus_cmd_3008_rs_t* rs;

//    const char *cname = "[OEM-ASUS_CMD_3008]";
    static unsigned char asus_postcode = 0;

	if(imsg->data_size > sizeof(oem_asus_cmd_3008_rq_t))
		return pp_bmc_router_resp_err(imsg, IPMI_ERR_REQ_DATA_LEN_LIMIT_EXCEEDED);
	if(imsg->data_size < sizeof(oem_asus_cmd_3008_rq_t))
		return pp_bmc_router_resp_err(imsg, IPMI_ERR_REQ_DATA_TRUNC);

    switch (rq->mode) {
    	case 0x00 :		//Read
    		if (rq->postcode != 0x00) {
				return pp_bmc_router_resp_err(imsg, IPMI_ERR_INVALID_DATA_FIELD);
    		};
    		break;
    	case 0x01 :		//Write
    		asus_postcode = rq->postcode;
    		break;
    	case 0x02 :		//Clear
    		asus_postcode = 0;
    		break;
    	default:
			return pp_bmc_router_resp_err(imsg, IPMI_ERR_INVALID_DATA_FIELD);
			break;
  	};

 	rs = pp_bmc_imsg_resp(imsg, IPMI_ERR_SUCCESS, sizeof(oem_asus_cmd_3008_rs_t));    
	rs->post_code = asus_postcode;
	return pp_bmc_router_send_msg(imsg);
}

/*
 * ---------------------------------------------------------------------------
 * Net Function : IPMI_NETFN_OEM_ASUS								0x30
 * Cmd Code     : IPMI_CMD_ASUS_READ_WRITE_SMART_FAN_STATUS			0x09
 * ---------------------------------------------------------------------------
 */

/*
 * ---------------------------------------------------------------------------
 * Net Function : IPMI_NETFN_OEM_ASUS								0x30
 * Cmd Code     : IPMI_CMD_ASUS_SET_OEM_DEVICE_ID					0x0A
 * ---------------------------------------------------------------------------
 */

/*
 * ---------------------------------------------------------------------------
 * Net Function : IPMI_NETFN_OEM_ASUS								0x30
 * Cmd Code     : IPMI_CMD_ASUS_TEST_MOTHERBOARD_SUPPORT_MODEL		0x0B
 * ---------------------------------------------------------------------------
 */
struct oem_asus_cmd_300B_rq_s {
    unsigned char mdl_str[0];
} __attribute__ ((packed));
typedef struct oem_asus_cmd_300B_rq_s oem_asus_cmd_300B_rq_t;

struct oem_asus_cmd_300B_rs_s {
    unsigned char sup_status;		//0:support, 1:not support
} __attribute__ ((packed));
typedef struct oem_asus_cmd_300B_rs_s oem_asus_cmd_300B_rs_t;

static int oem_asus_cmd_300B(imsg_t* imsg) {

    oem_asus_cmd_300B_rq_t* rq = (void*)imsg->data;
    oem_asus_cmd_300B_rs_t* rs;

    unsigned char resp_code = 1;
    const char *cname = "[OEM-ASUS_CMD_300B]";
    int fd, n;
    unsigned char sup_mdl_string[ASUS_MB_SUPPORT_MODEL_MAX_LENGTH];
    unsigned char mdl_string[ASUS_MB_SUPPORT_MODEL_MAX_LENGTH + 1];		//Add 1 more 0x00 byte for string function usage
	unsigned char entry_idx;

	if(imsg->data_size > 16)
		return pp_bmc_router_resp_err(imsg, IPMI_ERR_REQ_DATA_LEN_LIMIT_EXCEEDED);
	if(imsg->data_size < 1)
		return pp_bmc_router_resp_err(imsg, IPMI_ERR_REQ_DATA_TRUNC);

	memset(mdl_string, 0, sizeof(mdl_string));
	memcpy(mdl_string, rq->mdl_str, imsg->data_size);
	if (strlen(mdl_string) == 0) {
        pp_bmc_log_error("%s String length = 0", cname);
	} else {
        pp_bmc_log_info("%s String length = %d", cname, strlen(mdl_string));
        if (mbsupmdl_fileread_flag == 0) {		//First-time MB support model sring access
        	++mbsupmdl_fileread_flag;			//Access only once
		    /* open file */
		    assert(ASUS_MB_SUPPORT_MODEL_FILE_PATH);
		    if (0 > (fd = open(ASUS_MB_SUPPORT_MODEL_FILE_PATH, O_RDONLY))) {
		        pp_bmc_log_error("%s can't open file %s", cname, ASUS_MB_SUPPORT_MODEL_FILE_PATH);
			    return pp_bmc_router_resp_err(imsg, IPMI_ERR_ASUS_CMD_SPECIFIC_CODES_OEM_CMD_FAIL);
		    };
		    memset(asus_mb_sup_mdl_table, 0, sizeof(asus_mb_sup_mdl_table));
		    for (entry_idx = 0; entry_idx < ASUS_MB_SUPPORT_MODEL_MAX_NUMBER; entry_idx++, total_mdl_str_number++) {
			    memset(sup_mdl_string, 0, sizeof(sup_mdl_string));
			    /* read file */
			    n = read(fd, sup_mdl_string, ASUS_MB_SUPPORT_MODEL_MAX_LENGTH);
			    if ((n <= 0) || (n != ASUS_MB_SUPPORT_MODEL_MAX_LENGTH)) {
			    	break;
			    } else {
			    	memcpy(asus_mb_sup_mdl_table[entry_idx].model_name, sup_mdl_string, ASUS_MB_SUPPORT_MODEL_MAX_LENGTH);
			    };
			};
			close(fd);
		};
	    for (entry_idx = 0; entry_idx < total_mdl_str_number; entry_idx++) {
		    if (strcmp(mdl_string, asus_mb_sup_mdl_table[entry_idx].model_name) == 0) {
		        pp_bmc_log_info("%s String matched! idx=%d str='%s'", cname, (entry_idx+1), mdl_string);
		    	resp_code = 0;
		    	break;
		    };
		};
	};

 	rs = pp_bmc_imsg_resp(imsg, IPMI_ERR_SUCCESS, sizeof(oem_asus_cmd_300B_rs_t));    
	rs->sup_status = resp_code;
	return pp_bmc_router_send_msg(imsg);
}

/*
 * ---------------------------------------------------------------------------
 * Net Function : IPMI_NETFN_OEM_ASUS								0x30
 * Cmd Code     : IPMI_CMD_ASUS_GET_MOTHERBOARD_SUPPORT_MODEL_NAME	0x0C
 * ---------------------------------------------------------------------------
 */
struct oem_asus_cmd_300C_rq_s {
    unsigned char str_idx;
} __attribute__ ((packed));
typedef struct oem_asus_cmd_300C_rq_s oem_asus_cmd_300C_rq_t;

struct oem_asus_cmd_300C_rs_s {
    unsigned char total_sup_mdl_num;
    unsigned char mdl_str[ASUS_MB_SUPPORT_MODEL_MAX_LENGTH];
} __attribute__ ((packed));
typedef struct oem_asus_cmd_300C_rs_s oem_asus_cmd_300C_rs_t;

static int oem_asus_cmd_300C(imsg_t* imsg) {

    oem_asus_cmd_300C_rq_t* rq = (void*)imsg->data;
    oem_asus_cmd_300C_rs_t* rs;

    const char *cname = "[OEM-ASUS_CMD_300C]";
    int fd, n;
    unsigned char sup_mdl_string[ASUS_MB_SUPPORT_MODEL_MAX_LENGTH];
	unsigned char entry_idx;

	if(imsg->data_size > 1)
		return pp_bmc_router_resp_err(imsg, IPMI_ERR_REQ_DATA_LEN_LIMIT_EXCEEDED);
	if(imsg->data_size < 1)
		return pp_bmc_router_resp_err(imsg, IPMI_ERR_REQ_DATA_TRUNC);

    if (mbsupmdl_fileread_flag == 0) {		//First-time MB support model sring access
	   	++mbsupmdl_fileread_flag;			//Access only once
	    /* open file */
	    assert(ASUS_MB_SUPPORT_MODEL_FILE_PATH);
	    if (0 > (fd = open(ASUS_MB_SUPPORT_MODEL_FILE_PATH, O_RDONLY))) {
	        pp_bmc_log_error("%s can't open file %s", cname, ASUS_MB_SUPPORT_MODEL_FILE_PATH);
			    return pp_bmc_router_resp_err(imsg, IPMI_ERR_ASUS_CMD_SPECIFIC_CODES_OEM_CMD_FAIL);
	    };
	    memset(asus_mb_sup_mdl_table, 0, sizeof(asus_mb_sup_mdl_table));
	    for (entry_idx = 0; entry_idx < ASUS_MB_SUPPORT_MODEL_MAX_NUMBER; entry_idx++, total_mdl_str_number++) {
		    memset(sup_mdl_string, 0, sizeof(sup_mdl_string));
		    /* read file */
		    n = read(fd, sup_mdl_string, ASUS_MB_SUPPORT_MODEL_MAX_LENGTH);
		    if ((n <= 0) || (n != ASUS_MB_SUPPORT_MODEL_MAX_LENGTH)) {
		    	break;
		    } else {
		    	memcpy(asus_mb_sup_mdl_table[entry_idx].model_name, sup_mdl_string, ASUS_MB_SUPPORT_MODEL_MAX_LENGTH);
		    };
		};
		close(fd);
	};
//	if (rq->str_idx >= total_mdl_str_number) {
	if (rq->str_idx >= ASUS_MB_SUPPORT_MODEL_MAX_NUMBER) {
		return pp_bmc_router_resp_err(imsg, IPMI_ERR_PARAM_OUT_OF_RANGE);
	};

	entry_idx = rq->str_idx;
 	rs = pp_bmc_imsg_resp(imsg, IPMI_ERR_SUCCESS,
 			sizeof(oem_asus_cmd_300C_rs_t) - (ASUS_MB_SUPPORT_MODEL_MAX_LENGTH - strlen(asus_mb_sup_mdl_table[entry_idx].model_name)));
 	rs->total_sup_mdl_num = total_mdl_str_number;
	memcpy(rs->mdl_str, asus_mb_sup_mdl_table[entry_idx].model_name, strlen(asus_mb_sup_mdl_table[entry_idx].model_name));
	return pp_bmc_router_send_msg(imsg);
}

/*
 * ---------------------------------------------------------------------------
 * Net Function : IPMI_NETFN_OEM_ASUS								0x30
 * Cmd Code     : IPMI_CMD_ASUS_SET_MOTHERBOARD_SUPPORT_MODEL_NAME	0x0D
 * ---------------------------------------------------------------------------
 */
struct oem_asus_cmd_300D_rq_s {
    unsigned char str_idx;
    unsigned char mdl_str[0];
} __attribute__ ((packed));
typedef struct oem_asus_cmd_300D_rq_s oem_asus_cmd_300D_rq_t;

static int oem_asus_cmd_300D(imsg_t* imsg) {

    oem_asus_cmd_300D_rq_t* rq = (void*)imsg->data;

    const char *cname = "[OEM-ASUS_CMD_300D]";
    int fd, n;
    unsigned char sup_mdl_string[ASUS_MB_SUPPORT_MODEL_MAX_LENGTH];
	unsigned char entry_idx;
    unsigned char add_mdl_string[ASUS_MB_SUPPORT_MODEL_MAX_LENGTH];
	unsigned char clr_str[]={'C', 'L', 'R', 0x00};

	if(imsg->data_size > (1+ASUS_MB_SUPPORT_MODEL_MAX_LENGTH))
		return pp_bmc_router_resp_err(imsg, IPMI_ERR_REQ_DATA_LEN_LIMIT_EXCEEDED);
	if(imsg->data_size < (1+1))
		return pp_bmc_router_resp_err(imsg, IPMI_ERR_REQ_DATA_TRUNC);

    if (mbsupmdl_fileread_flag == 0) {		//First-time MB support model sring access
	   	++mbsupmdl_fileread_flag;			//Access only once
	    /* open file */
	    assert(ASUS_MB_SUPPORT_MODEL_FILE_PATH);
	    if (0 > (fd = open(ASUS_MB_SUPPORT_MODEL_FILE_PATH, O_RDONLY))) {
	        pp_bmc_log_error("%s can't open file %s", cname, ASUS_MB_SUPPORT_MODEL_FILE_PATH);
			    return pp_bmc_router_resp_err(imsg, IPMI_ERR_ASUS_CMD_SPECIFIC_CODES_OEM_CMD_FAIL);
	    };
	    memset(asus_mb_sup_mdl_table, 0, sizeof(asus_mb_sup_mdl_table));
	    for (entry_idx = 0; entry_idx < ASUS_MB_SUPPORT_MODEL_MAX_NUMBER; entry_idx++, total_mdl_str_number++) {
		    memset(sup_mdl_string, 0, sizeof(sup_mdl_string));
		    /* read file */
		    n = read(fd, sup_mdl_string, ASUS_MB_SUPPORT_MODEL_MAX_LENGTH);
		    if ((n <= 0) || (n != ASUS_MB_SUPPORT_MODEL_MAX_LENGTH)) {
		    	break;
		    } else {
		    	memcpy(asus_mb_sup_mdl_table[entry_idx].model_name, sup_mdl_string, ASUS_MB_SUPPORT_MODEL_MAX_LENGTH);
		    };
		};
		close(fd);
	};
	memset(add_mdl_string, 0 ,sizeof(add_mdl_string));
	memcpy(add_mdl_string, rq->mdl_str, (imsg->data_size - 1));
	if (rq->str_idx == 0xFF) {
		if (strcmp(add_mdl_string, clr_str) == 0) {
			memset(asus_mb_sup_mdl_table, 0, sizeof(asus_mb_sup_mdl_table));
			total_mdl_str_number = 0;
		} else {
		    return pp_bmc_router_resp_err(imsg, IPMI_ERR_INVALID_DATA_FIELD);
		};
	} else if (rq->str_idx >= ASUS_MB_SUPPORT_MODEL_MAX_NUMBER) {
	    return pp_bmc_router_resp_err(imsg, IPMI_ERR_INVALID_DATA_FIELD);
	} else if (strlen(add_mdl_string) == 0) {
        pp_bmc_log_error("%s String length = 0", cname);
	    return pp_bmc_router_resp_err(imsg, IPMI_ERR_INVALID_DATA_FIELD);
	} else if (rq->str_idx < total_mdl_str_number){		//Replace old model string
		memset(asus_mb_sup_mdl_table[rq->str_idx].model_name, 0 ,ASUS_MB_SUPPORT_MODEL_MAX_LENGTH);
		memcpy(asus_mb_sup_mdl_table[rq->str_idx].model_name, add_mdl_string, strlen(add_mdl_string));
        pp_bmc_log_info("%s String Add/Update idx=%d str='%s'", cname, ((rq->str_idx)+1), add_mdl_string);
    } else {											//Add new model string (force add in tail/last)
		memcpy(asus_mb_sup_mdl_table[total_mdl_str_number++].model_name, add_mdl_string, strlen(add_mdl_string));
        pp_bmc_log_info("%s String Add/Update idx=%d str='%s'", cname, total_mdl_str_number, add_mdl_string);
	};

	return pp_bmc_router_resp_err(imsg, IPMI_ERR_SUCCESS);
}

/*
 * ---------------------------------------------------------------------------
 * Net Function : IPMI_NETFN_OEM_ASUS								0x30
 * Cmd Code     : IPMI_CMD_ASUS_AUTO_SET_LAN_MAC_ADDRESS			0x0E	//Not implement
 * ---------------------------------------------------------------------------
 */

/*
 * ---------------------------------------------------------------------------
 * Net Function : IPMI_NETFN_OEM_ASUS								0x30
 * Cmd Code     : IPMI_CMD_ASUS_TEST_SET_BROADCOM_LAN_CONFIG_DATA	0x0F	//Not implement
 * ---------------------------------------------------------------------------
 */


/*
 * ---------------------------------------------------------------------------
 * Net Function : IPMI_NETFN_OEM_ASUS								0x30
 * Cmd Code     : IPMI_CMD_ASUS_VSC_RESET				0x20
 * ---------------------------------------------------------------------------
 */
struct oem_asus_cmd_3020_rq_s {
    unsigned char value;  	/* 0:Disable BMC scan I2C bus , 1:Enable BMC scan I2C bus */
} __attribute__ ((packed));
typedef struct oem_asus_cmd_3020_rq_s oem_asus_cmd_3020_rq_t;

static int oem_asus_cmd_3020(imsg_t* imsg) {

    oem_asus_cmd_3020_rq_t* rq = (void*)imsg->data;	

	if(imsg->data_size > sizeof(oem_asus_cmd_3020_rq_t))
		return pp_bmc_router_resp_err(imsg, IPMI_ERR_REQ_DATA_LEN_LIMIT_EXCEEDED);

	pp_vsc_reset(rq->value,VSC_RESET_PROGRAM_FPGA);


	return pp_bmc_router_resp_err(imsg, IPMI_ERR_SUCCESS);
}


/*
 * ---------------------------------------------------------------------------
 * Net Function : IPMI_NETFN_OEM_ASUS								0x30
 * Cmd Code     : IPMI_CMD_ASUS_EEPROM_READ_WRITE				0x21
 * ---------------------------------------------------------------------------
 */
struct oem_asus_cmd_3021_rq_s {
    unsigned char read_write;  
    unsigned char length;
    unsigned char offset_h;
    unsigned char offset_l;
    unsigned char buf;
} __attribute__ ((packed));

struct oem_asus_cmd_3021_rs_s {
    unsigned char eeprom_buf[64];
} __attribute__ ((packed));

typedef struct oem_asus_cmd_3021_rq_s oem_asus_cmd_3021_rq_t;
typedef struct oem_asus_cmd_3021_rs_s oem_asus_cmd_3021_rs_t;

static int oem_asus_cmd_3021(imsg_t* imsg) {

    unsigned int offset;
    oem_asus_cmd_3021_rq_t* rq = (void*)imsg->data;	
    oem_asus_cmd_3021_rs_t* rs;

	if(imsg->data_size < sizeof(oem_asus_cmd_3021_rq_t)-1)
		return pp_bmc_router_resp_err(imsg, IPMI_ERR_REQ_DATA_LEN_LIMIT_EXCEEDED);

	offset = (unsigned int) rq->offset_h;
 	offset = (offset << 8 )+rq->offset_l;
	if ( rq->read_write == 1 )	/* write eeprom */
	{
		pp_bmc_dev_oem_asus_write_eeprom(offset, rq->length, &rq->buf);
        }
        else				/* read eeprom */
	{
		pp_bmc_dev_oem_asus_read_eeprom(offset, rq->length, asus_eeprom_buf);
	 	rs = pp_bmc_imsg_resp(imsg, IPMI_ERR_SUCCESS, rq->length);    
		memcpy(rs->eeprom_buf, asus_eeprom_buf, rq->length);
		pp_bmc_log_warn("%x %x %x %x %x %x Get EEPROM Success", asus_eeprom_buf[0], asus_eeprom_buf[1], asus_eeprom_buf[2], asus_eeprom_buf[3], asus_eeprom_buf[4], asus_eeprom_buf[5]);
		return pp_bmc_router_send_msg(imsg);
	}


	return pp_bmc_router_resp_err(imsg, IPMI_ERR_SUCCESS);
}



/********************************************************************
 * Device command table
 */

static const dev_cmd_entry_t oem_asus_cmd_tab[] = {
    {
        .cmd_hndlr = oem_asus_cmd_3001,
        .netfn = IPMI_NETFN_OEM_ASUS,
		.cmd = IPMI_CMD_ASUS_READ_WRITE_SENSOR_VALUE,
        .min_data_size = sizeof(oem_asus_cmd_3001_rq_t),
		.min_priv_level = IPMI_PRIV_ADMIN
    },
    {
        .cmd_hndlr = oem_asus_cmd_3002,
        .netfn = IPMI_NETFN_OEM_ASUS,
		.cmd = IPMI_CMD_ASUS_ENABLE_BMC_SCAN_I2C_BUS,
        .min_data_size = sizeof(oem_asus_cmd_3002_rq_t),
		.min_priv_level = IPMI_PRIV_ADMIN
    },
//    {
//        .cmd_hndlr = oem_asus_cmd_3003,
//        .netfn = IPMI_NETFN_OEM_ASUS,
//		.cmd = IPMI_CMD_ASUS_WRITE_I2C_BYTE,
//        .min_data_size = sizeof(oem_asus_cmd_3003_rq_t),
//		.min_priv_level = IPMI_PRIV_ADMIN
//    },
//    {
//        .cmd_hndlr = oem_asus_cmd_3004,
//        .netfn = IPMI_NETFN_OEM_ASUS,
//		.cmd = IPMI_CMD_ASUS_GET_SENSOR_DATA_READY,
//        .min_data_size = sizeof(oem_asus_cmd_3004_rq_t),
//		.min_priv_level = IPMI_PRIV_ADMIN
//    },
//    {
//        .cmd_hndlr = oem_asus_cmd_3005,
//        .netfn = IPMI_NETFN_OEM_ASUS,
//		.cmd = IPMI_CMD_ASUS_GET_SUPPORT_MOTHERBOARD_TYPE,
//        .min_data_size = sizeof(oem_asus_cmd_3005_rq_t),
//		.min_priv_level = IPMI_PRIV_ADMIN
//    },
    {
        .cmd_hndlr = oem_asus_cmd_3006,
        .netfn = IPMI_NETFN_OEM_ASUS,
		.cmd = IPMI_CMD_ASUS_READ_ALL_SENSOR_VALUE,
        .min_data_size = sizeof(oem_asus_cmd_3006_rq_t),
		.min_priv_level = IPMI_PRIV_USER
    },
//    {
//        .cmd_hndlr = oem_asus_cmd_3007,
//        .netfn = IPMI_NETFN_OEM_ASUS,
//		.cmd = IPMI_CMD_ASUS_GET_OEM_DEVICE_ID,
//        .min_data_size = sizeof(oem_asus_cmd_3007_rq_t),
//		.min_priv_level = IPMI_PRIV_ADMIN
//    },
    {
        .cmd_hndlr = oem_asus_cmd_3008,
        .netfn = IPMI_NETFN_OEM_ASUS,
		.cmd = IPMI_CMD_ASUS_BIOS_POST_CODE,
        .min_data_size = sizeof(oem_asus_cmd_3008_rq_t),
		.min_priv_level = IPMI_PRIV_ADMIN
    },
//    {
//        .cmd_hndlr = oem_asus_cmd_3009,
//        .netfn = IPMI_NETFN_OEM_ASUS,
//		.cmd = IPMI_CMD_ASUS_READ_WRITE_SMART_FAN_STATUS,
//        .min_data_size = sizeof(oem_asus_cmd_3009_rq_t),
//		.min_priv_level = IPMI_PRIV_ADMIN
//    },
//    {
//        .cmd_hndlr = oem_asus_cmd_300A,
//        .netfn = IPMI_NETFN_OEM_ASUS,
//		.cmd = IPMI_CMD_ASUS_SET_OEM_DEVICE_ID,
//        .min_data_size = sizeof(oem_asus_cmd_300A_rq_t),
//		.min_priv_level = IPMI_PRIV_ADMIN
//    },
    {
        .cmd_hndlr = oem_asus_cmd_300B,
        .netfn = IPMI_NETFN_OEM_ASUS,
		.cmd = IPMI_CMD_ASUS_TEST_MOTHERBOARD_SUPPORT_MODEL,
        .min_data_size = sizeof(oem_asus_cmd_300B_rq_t),
		.min_priv_level = IPMI_PRIV_ADMIN
    },
    {
        .cmd_hndlr = oem_asus_cmd_300C,
        .netfn = IPMI_NETFN_OEM_ASUS,
		.cmd = IPMI_CMD_ASUS_GET_MOTHERBOARD_SUPPORT_MODEL_NAME,
        .min_data_size = sizeof(oem_asus_cmd_300C_rq_t),
		.min_priv_level = IPMI_PRIV_ADMIN
    },
    {
        .cmd_hndlr = oem_asus_cmd_300D,
        .netfn = IPMI_NETFN_OEM_ASUS,
		.cmd = IPMI_CMD_ASUS_SET_MOTHERBOARD_SUPPORT_MODEL_NAME,
        .min_data_size = sizeof(oem_asus_cmd_300D_rq_t),
		.min_priv_level = IPMI_PRIV_ADMIN
    },
//    {
//        .cmd_hndlr = oem_asus_cmd_300E,
//        .netfn = IPMI_NETFN_OEM_ASUS,
//		.cmd = IPMI_CMD_ASUS_AUTO_SET_LAN_MAC_ADDRESS,
//        .min_data_size = sizeof(oem_asus_cmd_300E_rq_t),
//		.min_priv_level = IPMI_PRIV_ADMIN
//    },
//    {
//        .cmd_hndlr = oem_asus_cmd_300F,
//        .netfn = IPMI_NETFN_OEM_ASUS,
//		.cmd = IPMI_CMD_ASUS_TEST_SET_BROADCOM_LAN_CONFIG_DATA,
//        .min_data_size = sizeof(oem_asus_cmd_300F_rq_t),
//		.min_priv_level = IPMI_PRIV_ADMIN
//    },
    {
        .cmd_hndlr = oem_asus_cmd_3020,
        .netfn = IPMI_NETFN_OEM_ASUS,
		.cmd = IPMI_CMD_ASUS_VSC_RESET,
        .min_data_size = sizeof(oem_asus_cmd_3020_rq_t),
		.min_priv_level = IPMI_PRIV_ADMIN
    },
    {
        .cmd_hndlr = oem_asus_cmd_3021,
        .netfn = IPMI_NETFN_OEM_ASUS,
		.cmd = IPMI_CMD_ASUS_EEPROM_READ_WRITE,
        .min_data_size = sizeof(oem_asus_cmd_3021_rq_t),
		.min_priv_level = IPMI_PRIV_ADMIN
    },
    { .cmd_hndlr = NULL }
};

/********************************************************************
 * Device c'tor/d'tor
 */

int pp_bmc_dev_oem_asus_init()
{
    /* register all entries of cmd tab */
    if (PP_FAILED(pp_bmc_core_reg_cmd_tab(oem_asus_cmd_tab))) return PP_ERR;

    pp_bmc_log_info("[OEM-ASUS] device started");
    return PP_SUC;
}

void pp_bmc_dev_oem_asus_cleanup()
{
    /* unregister all entries of cmd tab */
    pp_bmc_core_unreg_cmd_tab(oem_asus_cmd_tab);

    pp_bmc_log_info("[OEM-ASUS] device shut down");
}


//	ASUS OEM BMC POST INIT
//	1. Init message LED to 'OFF' state
//	2. Init locate LED to 'OFF' state
//	3. Switch MB COM2 for BMC usage		(Bus 5, SlaveAddress 0x30, W83601R GP12 = 0(O/P))
int pp_bmc_dev_oem_asus_bmc_post_init(void) {
	
	int	ret1, ret2;
	imsg_t imsg_pseudo;
	unsigned char imsg_data[2];
	const unsigned char *cname="[ASUS_BMC_POST_INIT]";
	int fd, n;
	unsigned char msgled_value = 0;

	asus_bmc_post_init_flag = 1;
	memset(&imsg_pseudo, 0, sizeof(imsg_pseudo));
	imsg_pseudo.data = imsg_data;
	imsg_pseudo.data_size = 2;
	imsg_data[0]=0x53;
    /* open file */
    assert(ASUS_BMC_POST_INIT_MSGLED_VALUE_FILE_PATH);
    if (0 > (fd = open(ASUS_BMC_POST_INIT_MSGLED_VALUE_FILE_PATH, O_RDONLY))) {
        pp_bmc_log_notice("%s can't open file %s", cname, ASUS_BMC_POST_INIT_MSGLED_VALUE_FILE_PATH);
    } else {
	    /* read file */
	    n = read(fd, &msgled_value, 1);
	    if ((n <= 0) || (n != 1)) {
	        pp_bmc_log_error("%s can't read from file %s", cname, ASUS_BMC_POST_INIT_MSGLED_VALUE_FILE_PATH);
	    } else if (msgled_value != 0) {
	    	msgled_value = 1;
	    };
		close(fd);
	};

	imsg_data[1] = msgled_value;
	ret1 = oem_asus_cmd_3001(&imsg_pseudo);	//Trun OFF message LED, and switch M/B COM2 for BMC usage
	imsg_data[0]=0x54;
	imsg_data[1]=0x00;
	ret2 = oem_asus_cmd_3001(&imsg_pseudo);	//Trun OFF locate LED
	asus_bmc_post_init_flag = 0;
	if ((ret1 == PP_SUC) && (ret2 == PP_SUC)) {
	    pp_bmc_log_info("%s Success", cname);
		return PP_SUC;
	} else {
	    pp_bmc_log_error("%s Fail", cname);
		return PP_ERR;
	};

}


//	ASUS BMC INIT TOP file read
int pp_bmc_dev_oem_asus_bmc_init_top_read(void) {
	
	unsigned char topfilepath[32];
	const unsigned char *cname="[ASUS_BMC_INIT_TOP_READ]";
	int fd, n;
	unsigned char *topfile_str = NULL;
	int	ret = PP_SUC;
	unsigned short top_file_length, top_file_read_length_left, top_file_read_offset, read_cnt;
	int i2c;
    int err = 0;
    unsigned char file_buf[TOP_FILE_MAX_LENGTH];
    int rx_burst_ret_cnt;
    unsigned short file_checksum, checksum = 0, idx;
	unsigned short seeprom_offset;

	topfile_str = pp_bmc_host_board_id_initial_filename();
	if (topfile_str == NULL) {
        pp_bmc_log_error("%s can't get the default top file name string", cname);
        return PP_ERR;
    };
	memset(topfilepath, 0, sizeof(topfilepath));
//	strcat(strcpy(topfilepath, ASUS_BMC_INIT_TOP_READ_FILE_FOLDER_PATH), topfile_str);
	strcpy(topfilepath, topfile_str);
	pp_bmc_log_info("%s Default top file path '%s'", cname, topfilepath);
	
//	Read SEEPROM content
    i2c = pp_i2c_open_bus(I2C_4_NUMBER, &err);
    if (err != PP_I2C_NO_ERROR) {
        pp_bmc_log_error("%s can't open i2c bus #%d", cname, I2C_4_NUMBER);
        ret = PP_ERR;
        goto finally_asus_bmc_init_top_read;
    };
	pp_bmc_log_notice("%s I2C R/W on '%s'", cname, pp_i2c_dev_by_hndl(i2c));
    pp_i2c_lock(i2c);

    seeprom_offset = TOP_FILE_SEEPROM_OFFSET;
	pp_i2c_tx_byte_data_core(i2c, ENABLE_DO_LOG_ERR, SEEPROM_SLAVE_ADDR,
			*((unsigned char *)(&seeprom_offset) + 1), *((unsigned char *)(&seeprom_offset)), &err);
	if (err != PP_I2C_NO_ERROR) {
	    pp_i2c_unlock(i2c);
	    pp_i2c_close(i2c);
        pp_bmc_log_error("%s SMBus access fail(seek to TOP_FILE_SEEPROM_OFFSET %d)", cname, TOP_FILE_SEEPROM_OFFSET);
        ret = PP_ERR;
        goto finally_asus_bmc_init_top_read;
	};
	pp_i2c_rx_burst_core(i2c, SEEPROM_SLAVE_ADDR, (unsigned char *)(&top_file_length), 2, &err);
	if (err != PP_I2C_NO_ERROR) {
	    pp_i2c_unlock(i2c);
	    pp_i2c_close(i2c);
        pp_bmc_log_error("%s SMBus read fail(top file length)", cname);
        ret = PP_ERR;
        goto finally_asus_bmc_init_top_read;
	};
	if (top_file_length > (TOP_FILE_MAX_LENGTH - 2)) {	// include 2 checksum bytes
		pp_i2c_unlock(i2c);
		pp_i2c_close(i2c);
		pp_bmc_log_error("%s Top file length exceeded %d(%d)", cname, (TOP_FILE_MAX_LENGTH - 2), top_file_length);
		ret = PP_ERR;
		goto finally_asus_bmc_init_top_read;
	};
    pp_bmc_log_info("%s Top file length %d", cname, (top_file_length - 4));
    //pp_i2c_rx_burst_core with 512 bytes maximum limit
    top_file_read_length_left = top_file_length - 2;
    top_file_read_offset = 0;
    while (top_file_read_length_left > 0) {
    	read_cnt = (top_file_read_length_left < 512) ? top_file_read_length_left : 512;
		rx_burst_ret_cnt = pp_i2c_rx_burst_core(i2c, SEEPROM_SLAVE_ADDR, &(file_buf[top_file_read_offset]), read_cnt, &err);
		if ((err != PP_I2C_NO_ERROR) || (rx_burst_ret_cnt != read_cnt)) {
		    pp_i2c_unlock(i2c);
		    pp_i2c_close(i2c);
	        pp_bmc_log_error("\n%s SMBus read fail at %d bytes offset", cname, (top_file_length - 2 - top_file_read_length_left));
	        ret = PP_ERR;
	        goto finally_asus_bmc_init_top_read;
		};
		top_file_read_length_left -= read_cnt;
		top_file_read_offset += read_cnt;
		printf("\rASUS topology file read %3d%%", (top_file_read_offset * 100 / (top_file_length -2)));
		fflush(stdout);
	};
	printf("\n");
    pp_i2c_unlock(i2c);
    pp_i2c_close(i2c);
// File checksum check
    file_checksum = *((unsigned short *)(&file_buf[0]));
    checksum = (unsigned char)(top_file_length & 0xFF) + (unsigned char)((top_file_length & 0xFF00) >> 8);
    for (idx = 0; idx < (top_file_length - 4); idx++) {
    	checksum += file_buf[idx + 2];
    };
    if ((unsigned short)(checksum + file_checksum) != 0) {
        pp_bmc_log_error("%s file checksum fail", cname);
//        pp_bmc_log_notice("%s file_checksum = 0x%4.4X", cname, file_checksum);
//        pp_bmc_log_notice("%s checksum = 0x%4.4X", cname, checksum);
        ret = PP_ERR;
        goto finally_asus_bmc_init_top_read;
    };
    pp_bmc_log_info("%s Top file checksum OK", cname);

    /* open file */
    assert(topfilepath);
    if (0 > (fd = open(topfilepath, O_WRONLY | O_CREAT | O_TRUNC, S_RW_RW_RW_))) {
        pp_bmc_log_error("%s can't open file %s", cname, topfilepath);
        ret = PP_ERR;
        goto finally_asus_bmc_init_top_read;
    };
    /* write file */
    n = write(fd, &(file_buf[2]), (top_file_length - 4));
    if ((n < 0) || (n != (top_file_length - 4))) {
        pp_bmc_log_error("%s write file %s error", cname, topfilepath);
        close(fd);
        ret = PP_ERR;
        goto finally_asus_bmc_init_top_read;
    };
	close(fd);

 finally_asus_bmc_init_top_read:
 	if (ret == PP_SUC) {
	    pp_bmc_log_info("%s Success", cname);
	    pp_bmc_log_info("%s Use new topology file from SEEPROM", cname);
		return PP_SUC;
	} else {
	    pp_bmc_log_error("%s Fail", cname);
	    pp_bmc_log_info("%s Use default topology file in F/W", cname);
		return PP_ERR;
	};
}

static char* asus_itoa_2u16(unsigned char value, unsigned char *string)
{
	unsigned char digit1, digit2;
	
	digit1 = (value & 0xF0) >> 4;
	digit2 = value & 0x0F;
	string[0] = (digit1>9) ?  ('A'+digit1-10) : ('0'+digit1);
	string[1] = (digit2>9) ?  ('A'+digit2-10) : ('0'+digit2);
	return string;
}
	
int pp_bmc_dev_oem_asus_get_mac(void)
{
	const unsigned char *cname="[ASUS_GET_MAC]";
	int i2c;
	int	ret = PP_SUC;
    int err = 0;
	unsigned char macadd[6];
	unsigned char macstr[32];
	int fd, n;
	unsigned short seeprom_offset;

	pp_bmc_log_warn("%s Get MAC", cname);
//	Read SEEPROM MAC content
    i2c = pp_i2c_open_bus(I2C_4_NUMBER, &err);
    if (err != PP_I2C_NO_ERROR) {
        pp_bmc_log_error("%s can't open i2c bus #%d", cname, I2C_4_NUMBER);
        ret = PP_ERR;
        goto finally_asus_get_mac;
    };
	pp_bmc_log_notice("%s I2C R/W on '%s'", cname, pp_i2c_dev_by_hndl(i2c));
    pp_i2c_lock(i2c);
    seeprom_offset = MAC_FILE_SEEPROM_OFFSET;
	pp_i2c_tx_byte_data_core(i2c, ENABLE_DO_LOG_ERR, SEEPROM_SLAVE_ADDR,
			*((unsigned char *)(&seeprom_offset) + 1), *((unsigned char *)(&seeprom_offset)), &err);
	if (err != PP_I2C_NO_ERROR) {
	    pp_i2c_unlock(i2c);
	    pp_i2c_close(i2c);
        pp_bmc_log_error("%s SMBus access fail(seek to MAC_FILE_SEEPROM_OFFSET %d)", cname, MAC_FILE_SEEPROM_OFFSET);
        ret = PP_ERR;
        goto finally_asus_get_mac;
	};
	pp_i2c_rx_burst_core(i2c, SEEPROM_SLAVE_ADDR, macadd, 6, &err);
	if (err != PP_I2C_NO_ERROR) {
	    pp_i2c_unlock(i2c);
	    pp_i2c_close(i2c);
        pp_bmc_log_error("%s SMBus read fail(top file length)", cname);
        ret = PP_ERR;
        goto finally_asus_get_mac;
	};
	if (macadd[0] == 0x00 && macadd[1] == 0x00 && macadd[2] == 0x00 && macadd[3] == 0x00 && macadd[4] == 0x00 && macadd[5] == 0x00) {
		pp_i2c_unlock(i2c);
		pp_i2c_close(i2c);
		pp_bmc_log_error("%s MAC address are all zeros", cname);
		ret = PP_ERR;
		goto finally_asus_get_mac;
	};
	if (macadd[0] == 0xFF && macadd[1] == 0xFF && macadd[2] == 0xFF && macadd[3] == 0xFF && macadd[4] == 0xFF && macadd[5] == 0xFF) {
		pp_i2c_unlock(i2c);
		pp_i2c_close(i2c);
		pp_bmc_log_error("%s MAC address are all 0xFF", cname);
		ret = PP_ERR;
		goto finally_asus_get_mac;
	};
    pp_i2c_unlock(i2c);
    pp_i2c_close(i2c);
	memset(macstr, 0, sizeof(macstr));
	strcat(asus_itoa_2u16(macadd[0], &(macstr[0])), ":");
	strcat(asus_itoa_2u16(macadd[1], &(macstr[3])), ":");
	strcat(asus_itoa_2u16(macadd[2], &(macstr[6])), ":");
	strcat(asus_itoa_2u16(macadd[3], &(macstr[9])), ":");
	strcat(asus_itoa_2u16(macadd[4], &(macstr[12])), ":");
	asus_itoa_2u16(macadd[5], &(macstr[15]));
    pp_bmc_log_warn("%s MAC = %s", cname, macstr);
	strcat(macstr, "\n");

/* open MAC new file */
    assert(ASUS_MAC_FILE_PATH);
    if (0 > (fd = open(ASUS_MAC_FILE_PATH, O_WRONLY | O_CREAT | O_TRUNC, S_RW_RW_RW_))) {
        pp_bmc_log_error("%s can't open file %s", cname, ASUS_MAC_FILE_PATH);
        ret = PP_ERR;
        goto finally_asus_get_mac;
    };
/* write MAC file */
    n = write(fd, macstr, strlen(macstr));
    if ((n < 0) || ((unsigned int)n != strlen(macstr))) {
        pp_bmc_log_error("%s write file %s error", cname, ASUS_MAC_FILE_PATH);
        close(fd);
        ret = PP_ERR;
        goto finally_asus_get_mac;
    };
	close(fd);

 finally_asus_get_mac:
 	if (ret == PP_SUC) {
	    pp_bmc_log_warn("%s Get MAC Success", cname);
		return PP_SUC;
	} else {
	    pp_bmc_log_error("%s Get MAC Fail", cname);
		return PP_ERR;
	};
}

int pp_bmc_dev_oem_asus_set_mac(void)
{
	const unsigned char *cname="[ASUS_SET_MAC]";
	const char *network_str="network.";
	const char *mac_str=".mac=";
    char command[256];
	int fd;
	FILE *file;
    char cfg_line_str[512], new_mac_setting_str[32];
	int status;
	unsigned char need_to_set_mac=0;
	unsigned char get_mac_str=0;
	

    pp_bmc_log_warn("%s Set MAC", cname);
    /* open KIRA configuration file*/
    assert(KIRA_CONFIG_FILE_PATH);
    if (0 > (fd = open(KIRA_CONFIG_FILE_PATH, O_RDONLY))) {
        pp_bmc_log_fatal("%s can't open file %s", cname, KIRA_CONFIG_FILE_PATH);
        return PP_ERR;
    };
    file = fdopen(fd, "r");
    /* check 'network.' string exist */
    while (1) {
    	if (fgets(cfg_line_str, sizeof(cfg_line_str), file) == NULL) {	//Some error happened
	        pp_bmc_log_critical("%s can't find string '%s'", cname, network_str);
	        need_to_set_mac = 1;
	        break;
	    } else {
	    	if (strncmp(cfg_line_str, network_str, strlen(network_str)) == 0) {
	    		cfg_line_str[strlen(cfg_line_str)-1] = 0x00;	//cut the trailing new line character
			    pp_bmc_log_info("%s Get '%s' in string (%s)", cname, network_str, cfg_line_str);
			    /* check '.mac=' string exist */
		    	if (strncmp(&(cfg_line_str[strlen(network_str)-1]), mac_str, strlen(mac_str)) == 0) {
				    pp_bmc_log_info("%s Get '%s' in string (%s)", cname, mac_str, cfg_line_str);
				    get_mac_str = 1;
				    strncpy(cfg_line_str, &(cfg_line_str[strlen(network_str)-1]), strlen(cfg_line_str)-strlen(network_str)+1);
		    		break;
		    	};
			    while (1) {
			    	if (fgets(cfg_line_str, sizeof(cfg_line_str)-1, file) == NULL) {	//Some error happened
				        pp_bmc_log_critical("%s can't find string '%s'", cname, mac_str);
				        need_to_set_mac = 1;
				        break;
				    } else if (strncmp(cfg_line_str, mac_str, 1) == 0) {
				    	if (strncmp(cfg_line_str, mac_str, strlen(mac_str)) == 0) {
				    		cfg_line_str[strlen(cfg_line_str)-1] = 0x00;	//cut the trailing new line character
						    pp_bmc_log_info("%s Get '%s' in string (%s)", cname, mac_str, cfg_line_str);
						    get_mac_str = 1;
				    		break;
				    	};
				    } else {
				    	need_to_set_mac = 1;
				    	break;
				    };
			    };
	    		break;
	    	};
	    };
    };
	close(fd);

   	if (get_mac_str == 1) {
	    /* open new MAC setting file*/
	    assert(ASUS_MAC_FILE_PATH);
	    if (0 > (fd = open(ASUS_MAC_FILE_PATH, O_RDONLY))) {
	        pp_bmc_log_error("%s can't open file %s (New MAC not set)", cname, ASUS_MAC_FILE_PATH);
	        return PP_SUC;
	    };
    	file = fdopen(fd, "r");
	    /* compare MAC address setting */
    	if (fgets(new_mac_setting_str, sizeof(new_mac_setting_str), file) == NULL) {	//Some error happened
	        pp_bmc_log_error("%s can't get new MAC setting string from file %s", cname, ASUS_MAC_FILE_PATH);
	        close(fd);
	        return PP_SUC;
	    };
	    if (strncmp(&(cfg_line_str[strlen(mac_str)]), new_mac_setting_str, strlen(new_mac_setting_str)-1) == 0) {
		    pp_bmc_log_warn("%s new MAC address matched with the configuation", cname);
	    } else {
		    pp_bmc_log_warn("%s new MAC address setup", cname);
	    	need_to_set_mac = 1;
	    };
        close(fd);
   	};
   	
   	if (need_to_set_mac == 1) {
	   	snprintf(command, sizeof(command), "eric_config -changemac <%s\n", ASUS_MAC_FILE_PATH);
	   	if ((status = pp_system(command)) == -1 || WEXITSTATUS(status) != 0) {
		    pp_log("%s(): Set MAC to configuration file failed.\n", ___F);
		    return PP_ERR;
		};
	};

	return PP_SUC;
}



int pp_bmc_dev_oem_asus_read_eeprom(unsigned int seeprom_offset, unsigned char length, unsigned char *eeprom_buf)
{
	const unsigned char *cname="[ASUS_READ_EEPROM]";
	int i2c;
	int	ret = PP_SUC;
    int err = 0;
	unsigned char buf[64];

	pp_bmc_log_warn("%s READ EEPROM %x %x %d", cname, seeprom_offset>>8, seeprom_offset & 0xff, length);
//	Read SEEPROM MAC content
    i2c = pp_i2c_open_bus(I2C_4_NUMBER, &err);
    if (err != PP_I2C_NO_ERROR) {
        pp_bmc_log_error("%s can't open i2c bus #%d", cname, I2C_4_NUMBER);
        ret = PP_ERR;
        goto finally_asus_read_eeprom;
    };
	pp_bmc_log_notice("%s I2C R/W on '%s'", cname, pp_i2c_dev_by_hndl(i2c));
    pp_i2c_lock(i2c);

//	pp_i2c_tx_byte_data_core(i2c, ENABLE_DO_LOG_ERR, SEEPROM_SLAVE_ADDR,
//		seeprom_offset>>8, seeprom_offset & 0xff, &err);
	pp_i2c_tx_byte_data_core(i2c, ENABLE_DO_LOG_ERR, SEEPROM_SLAVE_ADDR,
			*((unsigned char *)(&seeprom_offset) + 1), *((unsigned char *)(&seeprom_offset)), &err);
	if (err != PP_I2C_NO_ERROR) {
	    pp_i2c_unlock(i2c);
	    pp_i2c_close(i2c);
        pp_bmc_log_error("%s SMBus access fail(seek to MAC_FILE_SEEPROM_OFFSET %d)", cname, MAC_FILE_SEEPROM_OFFSET);
        ret = PP_ERR;
        goto finally_asus_read_eeprom;

	};
	pp_i2c_rx_burst_core(i2c, SEEPROM_SLAVE_ADDR, buf, length, &err);
	if (err != PP_I2C_NO_ERROR) {
	    pp_i2c_unlock(i2c);
	    pp_i2c_close(i2c);
        pp_bmc_log_error("%s SMBus read fail(top file length)", cname);
        ret = PP_ERR;
        goto finally_asus_read_eeprom;
	};

    pp_i2c_unlock(i2c);
    pp_i2c_close(i2c);

 finally_asus_read_eeprom:
 	if (ret == PP_SUC) {
	    memcpy(eeprom_buf, buf, length);
	    memcpy(asus_eeprom_buf, buf, length);
	    pp_bmc_log_warn("%s %x %x %x %x %x %x Get EEPROM Success", cname, eeprom_buf[0], eeprom_buf[1], eeprom_buf[2], eeprom_buf[3], eeprom_buf[4], eeprom_buf[5]);
	    pp_bmc_log_warn("%s %x %x %x %x %x %x Get EEPROM Success", cname, buf[0], buf[1], buf[2], buf[3], buf[4], buf[5]);
		return PP_SUC;
	} else {
	    pp_bmc_log_error("%s Get EEPROM Fail", cname);
		return PP_ERR;
	};
}

int pp_bmc_dev_oem_asus_write_eeprom(unsigned int seeprom_offset, unsigned char length, unsigned char *eeprom_buf)
{
	const unsigned char *cname="[ASUS_WRITE_EEPROM]";
	int i2c;
	int	ret = PP_SUC;
    int err = 0;
	unsigned char tmp_buf[64];

	memcpy(tmp_buf+2, eeprom_buf, length);
	tmp_buf[0]=seeprom_offset>>8;
	tmp_buf[1]=seeprom_offset&0xff;

	pp_bmc_log_warn("%s Write EEPROM %x %x %d", cname, seeprom_offset>>8, seeprom_offset & 0xff, length);
//	Read SEEPROM MAC content
    i2c = pp_i2c_open_bus(I2C_4_NUMBER, &err);
    if (err != PP_I2C_NO_ERROR) {
        pp_bmc_log_error("%s can't open i2c bus #%d", cname, I2C_4_NUMBER);
        ret = PP_ERR;
        goto finally_asus_write_eeprom;
    };
	pp_bmc_log_notice("%s I2C R/W on '%s'", cname, pp_i2c_dev_by_hndl(i2c));
    pp_i2c_lock(i2c);

	pp_i2c_tx_burst_core(i2c, SEEPROM_SLAVE_ADDR, tmp_buf, length+2, &err);
	if (err != PP_I2C_NO_ERROR) {
	    pp_i2c_unlock(i2c);
	    pp_i2c_close(i2c);
        pp_bmc_log_error("%s SMBus read fail(top file length)", cname);
        ret = PP_ERR;
        goto finally_asus_write_eeprom;
	};

    pp_i2c_unlock(i2c);
    pp_i2c_close(i2c);

 finally_asus_write_eeprom:
 	if (ret == PP_SUC) {
	    pp_bmc_log_warn("%s Get EEPROM Success", cname);
pp_bmc_log_warn("%s %x %x %x %x %x %x Get EEPROM Success", cname, eeprom_buf[0], eeprom_buf[1], eeprom_buf[2], eeprom_buf[3], eeprom_buf[4], eeprom_buf[5]);

		return PP_SUC;
	} else {
	    pp_bmc_log_error("%s Get EEPROM Fail", cname);
		return PP_ERR;
	};

}
