/*
 * <3BSD notice header>
 *
 * Copyright (C) 2003-2008 Raritan, Inc. <http://www.raritan.com/>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 * 
 *     * Neither the name of [original copyright holder] nor the names of
 *       its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written
 *       permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * \file debug.h
 *
 * Description: Logging Macros
 */

#ifndef _PP_BMC_DEBUG_H_
#define _PP_BMC_DEBUG_H_
#include <stdarg.h>
#include <sys/time.h>
#include <time.h>

/**
 * Debugging can be controlled with the following compiler switches 
 *
 * PP_BMC_DEBUG_LEVEL xx   enable debugging on level xx
 *                         higher lovlevel = more messages
 *                         debugging is disabled if undefined
 *   8  debug
 *   7  info
 *   6  notice
 *   5  warn
 *   4  error
 *   3  critical
 *   2  alert
 *   1  fatal
 *   0  disable debugging
 */

#ifdef PP_BUILD_TYPE_FINAL
#  ifdef PP_BMC_DEBUG_LEVEL
#    undef PP_BMC_DEBUG_LEVEL
#  endif
#  define PP_BMC_DEBUG_LEVEL 5
#else 
#  ifndef  PP_BMC_DEBUG_LEVEL
#    define PP_BMC_DEBUG_LEVEL 5
#  endif
#endif


#define PP_BMC_LOG_DEBUG    8
#define PP_BMC_LOG_INFO     7
#define PP_BMC_LOG_NOTICE   6
#define PP_BMC_LOG_WARN     5
#define PP_BMC_LOG_ERROR    4
#define PP_BMC_LOG_CRITICAL 3
#define PP_BMC_LOG_ALERT    2
#define PP_BMC_LOG_FATAL    1

/** Don't use this macro directly */
#if PP_BMC_DEBUG_LEVEL>=1

void pp_bmc_vlog(int level, const char* format, va_list ap);
void pp_bmc_log(int level, const char* format, ...);
void pp_bmc_plog(int lovel, const char* format, ...);

#endif

#if PP_BMC_DEBUG_LEVEL>=8
#define pp_bmc_log_debug(logstring, args...) \
    pp_bmc_log(PP_BMC_LOG_DEBUG, logstring, ##args)
#define pp_bmc_log_pdebug(logstring, args...) \
    pp_bmc_plog(PP_BMC_LOG_DEBUG, logstring, ##args)
#else
#define pp_bmc_log_debug(logstring, args...)
#define pp_bmc_log_pdebug(logstring, args...)
#endif

#if PP_BMC_DEBUG_LEVEL>=7
#define pp_bmc_log_info(logstring, args...) \
    pp_bmc_log(PP_BMC_LOG_INFO, logstring, ##args)
#define pp_bmc_log_pinfo(logstring, args...) \
    pp_bmc_plog(PP_BMC_LOG_INFO, logstring, ##args)
#else
#define pp_bmc_log_info(logstring, args...)
#define pp_bmc_log_pinfo(logstring, args...)
#endif

#if PP_BMC_DEBUG_LEVEL>=6
#define pp_bmc_log_notice(logstring, args...)  \
    pp_bmc_log(PP_BMC_LOG_NOTICE, logstring, ##args)
#define pp_bmc_log_pnotice(logstring, args...) \
    pp_bmc_plog(PP_BMC_LOG_NOTICE, logstring, ##args)
#else
#define pp_bmc_log_notice(logstring, args...)
#define pp_bmc_log_pnotice(logstring, args...)
#endif

#if PP_BMC_DEBUG_LEVEL>=5
#define pp_bmc_log_warn(logstring, args...) \
    pp_bmc_log(PP_BMC_LOG_WARN, logstring, ##args)
#define pp_bmc_log_pwarn(logstring, args...) \
    pp_bmc_plog(PP_BMC_LOG_WARN, logstring, ##args)
#else
#define pp_bmc_log_warn(logstring, args...)
#define pp_bmc_log_pwarn(logstring, args...)
#endif

#if PP_BMC_DEBUG_LEVEL>=4
#define pp_bmc_log_error(logstring, args...) \
    pp_bmc_log(PP_BMC_LOG_ERROR, logstring, ##args)
#define pp_bmc_log_perror(logstring, args...) \
    pp_bmc_plog(PP_BMC_LOG_ERROR, logstring, ##args)
#else
#define pp_bmc_log_error(logstring, args...)
#define pp_bmc_log_perror(logstring, args...)
#endif

#if PP_BMC_DEBUG_LEVEL>=3
#define pp_bmc_log_critical(logstring, args...) \
    pp_bmc_log(PP_BMC_LOG_CRITICAL, logstring, ##args)
#define pp_bmc_log_pcritical(logstring, args...) \
    pp_bmc_plog(PP_BMC_LOG_CRITICAL, logstring, ##args)
#else
#define pp_bmc_log_critical(logstring, args...)
#define pp_bmc_log_pcritical(logstring, args...)
#endif

#if PP_BMC_DEBUG_LEVEL>=2
#define pp_bmc_log_alert(logstring, args...) \
    pp_bmc_log(PP_BMC_LOG_ALERT, logstring, ##args)
#define pp_bmc_log_palert(logstring, args...) \
    pp_bmc_plog(PP_BMC_LOG_ALERT, logstring, ##args)
#else
#define pp_bmc_log_alert(logstring, args...)
#define pp_bmc_log_palert(logstring, args...)
#endif

#if PP_BMC_DEBUG_LEVEL>=1
#define pp_bmc_log_fatal(logstring, args...) \
    pp_bmc_log(PP_BMC_LOG_FATAL, logstring, ##args)
#define pp_bmc_log_pfatal(logstring, args...) \
    pp_bmc_plog(PP_BMC_LOG_FATAL, logstring, ##args)
#else
#define pp_bmc_log_fatal(logstring, args...)
#define pp_bmc_log_pfatal(logstring, args...)
#endif

void pp_bmc_printbuf(const unsigned char* prefix,
                     const unsigned char* buf, int buf_len);

#endif

