/*
 * <3BSD notice header>
 *
 * Copyright (C) 2003-2008 Raritan, Inc. <http://www.raritan.com/>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 * 
 *     * Neither the name of [original copyright holder] nor the names of
 *       its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written
 *       permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * adt7463_sensor.h
 *
 * "Super class" for all ADT7463 sensors
 */

#ifndef __ADT7463_SENSOR_H
#define __ADT7463_SENSOR_H

#include <pp/vector.h>
#include <pp/base.h>

#include <pp/bmc/topo_base_obj.h>
#include <pp/bmc/tp_i2c_chip.h>


/* ppdoc:
 * name:   ADT7463 sensor
 * tpname: ADT7463sens
 * desc:   ADT7463 sensor object, dissolves to different sensors depending
 * desc:   on the arguments. Relies on an ADT7463 chip object.
 * arg 0: [OBJ(I2C_CHIP)] chip:  i2c chip to use, must be an ADT7463 chip object
 * arg 1: [INT] sensor_no:       sensor number within the chip, whereas:
 * arg 1:                         0: 5v
 * arg 1:                         1: 2_5v
 * arg 1:                         2: 12v
 * arg 1:                         3: vcc
 * arg 1:                         4: vccp
 * arg 1:                         5: tach1
 * arg 1:                         6: tach2
 * arg 1:                         7: tach3
 * arg 1:                         8: tach4
 * arg 1:                         9: temp_local
 * arg 1:                        10: temp_remote1
 * arg 1:                        11: temp_remote2
 * arg 1:                        12: vid0	(HD plug in/out)
 * arg 1:                        13: vid1	(HD plug in/out)
 * arg 1:                        14: vid2	(HD plug in/out)
 * arg 1:                        15: vid3	(HD plug in/out)
 * arg 1:                        16: vid4	(HD plug in/out)
 * arg 1:                        17: vid5	(HD plug in/out)
 * arg 1:                        18: pwm1	(LED control)
 * arg 1:                                        output 0: drive out high for green LED(OK)
 * arg 1:                                        output 1: drive out low for red LED(FAIL)
 * arg 2: [OBJ(COND)] scan_cond: (optional) scan_condition, sensor won't be
 * arg 2:                        read if condition is false
 * return: [OBJ(SENS_DEV)]
 */
pp_tp_obj_t* pp_adt7463_sensor_ctor(const char* id, vector_t* args);

/*#define PP_SENSOR_ADT7463_MAX_SENSOR_NUM 18*/
#define PP_SENSOR_ADT7463_MAX_SENSOR_NUM 26

#endif /* __ADT7463_SENSOR_H */
