/**
 * some file header - todo
 * 
 */ 

#include <pp/selector.h>

#include <pp/bmc/debug.h>
#include <pp/bmc/topo_factory.h>

#include "drivers/blackford_mch_chip.h"


/* internal prototypes */
static
pp_tp_obj_t* blackford_mch_chip_init(const unsigned char* id,
                               pp_tp_i2c_comdev_t* i2cdev,
                               unsigned char i2caddr,
                               pp_tp_cond_t* scan);
static void pp_tp_blackford_mch_chip_cleanup(pp_tp_blackford_mch_chip_t* this);


pp_tp_obj_t* pp_blackford_mch_chip_ctor(const char* id, vector_t* args) {
    pp_strstream_t err = PP_STRSTREAM_INITIALIZER;

    pp_tp_i2c_comdev_t* i2cdev;
    int i2caddr;
    pp_tp_cond_t* scan;
      
    if (pp_tp_arg_scanf(args, 0, &err, "o<i>do<sc>",
                        &i2cdev, &i2caddr, &scan) != 3)
    {
        pp_bmc_log_error("'%s' failed: %s (%s)",
                        id, strerror(errno), pp_strstream_buf(&err));
        return NULL;
    }
    
    return blackford_mch_chip_init(id, i2cdev, i2caddr, scan);
}

static
pp_tp_obj_t* blackford_mch_chip_init(const unsigned char* id,
                               pp_tp_i2c_comdev_t* i2cdev,
                               unsigned char i2caddr,
                               pp_tp_cond_t* scan)
{   
    pp_tp_blackford_mch_chip_t* this;

    this = malloc(sizeof(pp_tp_blackford_mch_chip_t));
    memset(this, 0, sizeof(pp_tp_blackford_mch_chip_t));
    
    pp_tp_i2c_chip_init((pp_tp_i2c_chip_t*)this, PP_TP_I2C_CHIP,
                        id, (void*)pp_tp_blackford_mch_chip_cleanup,
                        "blackford_mch", i2cdev, 
                        i2caddr, scan,
                        NULL,
                        NULL);

    pp_bmc_log_info("[BLACKFORD_MCH] '%s' initialized", id);
    
    return (pp_tp_obj_t*)this;
}                                          

static void pp_tp_blackford_mch_chip_cleanup(pp_tp_blackford_mch_chip_t* this) {
    pp_bmc_log_info("[BLACKFORD_MCH] cleaning up chip '%s'", ((pp_tp_obj_t*)this)->id);

    pp_tp_i2c_chip_cleanup((pp_tp_i2c_chip_t*) this);
    free (this);
}

