/**
 * pcf8574_sensor.h
 *
 * "Super class" for all PCF8574 sensors
 * 
 */

#ifndef __PCF8574_SENSOR_H
#define __PCF8574_SENSOR_H

#include <pp/vector.h>
#include <pp/base.h>

#include <pp/bmc/topo_base_obj.h>
#include <pp/bmc/tp_i2c_chip.h>


/* ppdoc:
 * name:   PCF8574 sensor
 * tpname: PCF8574sens
 * desc:   PCF8574 sensor object, dissolves to different sensors depending
 * desc:   on the arguments. Relies on an PCF8574 chip object.
 * arg 0: [OBJ(I2C_CHIP)] chip:  i2c chip to use, must be an PCF8574 chip object
 * arg 1: [INT] sensor_no:       sensor number within the chip, whereas:
 * arg 1:                         0: P0
 * arg 1:                         1: P1
 * arg 1:                         2: P2
 * arg 1:                         3: P3
 * arg 1:                         4: P4
 * arg 1:                         5: P5
 * arg 1:                         6: P6
 * arg 1:                         7: P7
 * arg 1:                         8: all
 * arg 2: [OBJ(COND)] scan_cond: (optional) scan_condition, sensor won't be
 * arg 2:                        read if condition is false
 * return: [OBJ(SENS_DEV)]
 */
pp_tp_obj_t* pp_pcf8574_sensor_ctor(const char* id, vector_t* args);

#define PP_SENSOR_PCF8574_MAX_SENSOR_NUM 8

#endif /* __PCF8574_SENSOR_H */
