/*
 * <3BSD notice header>
 *
 * Copyright (C) 2003-2008 Raritan, Inc. <http://www.raritan.com/>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 * 
 *     * Neither the name of [original copyright holder] nor the names of
 *       its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written
 *       permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * topo_ctors.h
 *
 * This file defines the preefined Topology Descriptor lists.
 * 
 * Any code that plugs into the topology framework must be
 * registered here (unless we got a dymanic method to do that)!
 */

#include "topo_parser_impl.h"
#include "topo_ctors.h"
#include <pp/bmc/topo_base_obj.h>

#include <pp/bmc/tp_ipmi_sens.h>
#include <pp/bmc/tp_muxed_scanner.h>
#include <pp/bmc/tp_gpio_sens.h>
#include <pp/bmc/tp_gpio_multi_sens.h>
#include <pp/bmc/tp_gpio_bulk_sens.h>
#include <pp/bmc/tp_gpio_act.h>
#include <pp/bmc/tp_gpio_multi_act.h>
#include <pp/bmc/tp_gpio_loop.h>
#include <pp/bmc/tp_sdr.h>
#include <pp/bmc/tp_static_cond.h>
#include <pp/bmc/tp_delayed_cond.h>
#include <pp/bmc/tp_i2c_alive_cond.h>
#include <pp/bmc/tp_sens_has_value_cond.h>
#include <pp/bmc/tp_and_cond.h>
#include <pp/bmc/tp_acpi_power_state_opma.h>
#include <pp/bmc/tp_acpi_power_state_gpio.h>
#include <pp/bmc/tp_event_collect_ctrl.h>
#include <pp/bmc/tp_event_collect_pwm_ctrl.h>
#include "drivers/dummy_sensors.h"
#include "drivers/pp_i2c_dev.h"
#include "drivers/pp_fpga_gpio_dev.h"
#include "drivers/pp_tach_sens.h"
#include "drivers/pp_fpga_pwm_dev.h"
#include "drivers/pp_gpio_disc_sens.h"
#include "drivers/pp_powersupply_sens.h"
#include "drivers/pca9555_gpio_dev.h"
#include "drivers/pca9556_gpio_dev.h"
#include "drivers/max7312_gpio_dev.h"
#include "drivers/pca9542_i2c_mux.h"
#include "drivers/pca9544_i2c_mux.h"
#include "drivers/pca9548_i2c_mux.h"
#include "drivers/ltc4305_i2c_mux.h"
#include "drivers/ltc4306_i2c_mux.h"
#include "drivers/adm1031_sensor.h"
#include "drivers/adm1029.h"
#include "drivers/adm1026.h"
#include "drivers/adm1024.h"
#include "drivers/ads7830_sensor.h"
#include "drivers/w83791d.h"
#include "drivers/w83791d_sensor.h"
#include "drivers/w83791d_gpio.h"
#include "drivers/w83791d_pwm.h"
#include "drivers/lm75_sensor.h"
#include "drivers/adt7463_chip.h"
#include "drivers/adt7463_sensor.h"
#include "drivers/adt7463_pwm.h"
#include "drivers/max6695_sensor.h"
#include "drivers/max6649_sensor.h"
#include "drivers/max1037_adc_sensor.h"
#include "drivers/max6681_sensor.h"
#include "drivers/msi_smx.h"
#include "drivers/msi_fan_ctrl.h"
#include "drivers/opma_smx.h"
#include "drivers/smi_smx.h"
#include "drivers/warthog_fan_ctrl.h"
#include "drivers/cheyenne_fan_ctrl.h"
#include "drivers/linear_fan_ctrl.h"
#include "drivers/adc_kira_sensor.h"
#include "drivers/rccy8.h"
#include "drivers/rpc_pwr_sens.h"
#include "drivers/rpc_disp_ctrl.h"
#include "drivers/rpc_led_ctrl.h"
#include "drivers/w83792d.h"
#include "drivers/w83792d_sensor.h"
#include "drivers/w83792d_clr_intrusion.h"
#include "drivers/lm94.h"
#include "drivers/lm94_sensor.h"
#include "drivers/w83793g.h"
#include "drivers/w83793g_sensor.h"
#include "drivers/w83793g_clr_intrusion.h"
#include "drivers/rc_rs485_dev.h"
#include "drivers/rc_cy8c26.h"
#include "drivers/pca9545_i2c_mux.h"
#include "drivers/dummy_smx.h"
#include "drivers/emc6d103s.h"
#include "drivers/pcf8574_chip.h"
#include "drivers/pcf8574_sensor.h"
#include "drivers/blackford_mch_chip.h"
#include "drivers/blackford_mch_sensor.h"
#include "drivers/asusgeneral.h"
#include "drivers/adt7476.h"
#include "drivers/w83627hf.h"
#include "drivers/w83627dhg.h"
#include "drivers/w83627dhg_1.h"
#include "drivers/w83627dhg_2.h"


pp_tp_obj_ctor_def_t pp_tp_ctors_basic[] = {
    /* chips */
    { "ADM1029",              	pp_sensor_adm1029_ctor },
    { "ADM1026",              	pp_sensor_adm1026_ctor },
    { "ADM1024",              	pp_sensor_adm1024_ctor },
    { "ADT7463",		pp_adt7463_chip_ctor },
    { "W83791D",              	pp_sensor_w83791d_ctor },
    { "RCCY8",                  pp_sensor_rccy8_ctor },
    { "W83792D",              	pp_sensor_w83792d_ctor },
    { "LM94",              	pp_lm94_ctor },
    { "W83793G",              	pp_sensor_w83793g_ctor },
    { "RCCY8C26",               pp_rc_cy8c26_chip_ctor },
    { "EMC6D103S",              pp_sensor_emc6d103s_ctor },
    { "PCF8574",		pp_pcf8574_chip_ctor },
    { "BLACKFORD",		pp_blackford_mch_chip_ctor },
    { "ASUSGENERAL",              	pp_sensor_asusgeneral_ctor },
	
    /* devices */
    { "I2C",                    pp_i2c_dev_ctor },
    { "I2CMux9542",		pp_i2c_mux_pca9542_ctor },
    { "I2CMux9544",		pp_i2c_mux_pca9544_ctor },
    { "I2CMux9545",		pp_i2c_mux_pca9545_ctor },
    { "I2CMux9548",		pp_i2c_mux_pca9548_ctor },
    { "I2CMux4305",		pp_i2c_mux_ltc4305_ctor },
    { "I2CMux4306",		pp_i2c_mux_ltc4306_ctor },
    { "RS485",                  pp_rc_rs485_dev_ctor },
    { "FpgaGpio",		pp_fpga_gpio_dev_ctor },
    { "PCA9554Gpio",		pp_pca9556_gpio_dev_ctor }, //alias for 9556
    { "PCA9555Gpio",		pp_pca9555_gpio_dev_ctor },
    { "PCA9556Gpio",		pp_pca9556_gpio_dev_ctor },
    { "MAX7312Gpio",		pp_max7312_gpio_dev_ctor },
    { "ADM1029Gpio",            pp_sensor_adm1029_gpio_ctor },
    { "W83791DGpio",            pp_sensor_w83791d_gpio_ctor },
    { "RCCY8C26Gpio",           pp_rc_cy8c26_gpio_ctor },
    

    /* sensors */
    { "ISens",			pp_tp_ipmi_sens_ctor },
    { "ISensFullSDR",		pp_tp_ipmi_sens_full_sdr_ctor },
    { "ISensCompactSDR",	pp_tp_ipmi_sens_compact_sdr_ctor },
    { "BMCInfoSDR",		pp_tp_bmc_info_sdr_ctor },
    { "FRUInfoSDR",		pp_tp_asus_fru_info_sdr_ctor },
    { "DummySens",		pp_sensor_dummy_ctor },
    { "GpioSens",		pp_tp_gpio_sens_ctor },
    { "TachSens",		pp_tach_sens_ctor },    
    { "GpioMultiSens",		pp_tp_gpio_multi_sens_ctor },
    { "GpioBulkSens",           pp_tp_gpio_bulk_sens_ctor },
    { "GpioDiscSens",		pp_gpio_disc_sens_ctor },
    { "PowersupplySens",	pp_powersupply_sens_ctor },
    { "AcpiPowerStateOpma",	pp_tp_acpi_power_state_opma_ctor },
    { "AcpiPowerStateGpio",	pp_tp_acpi_power_state_gpio_ctor },
    { "ADT7463sens",            pp_adt7463_sensor_ctor },
    { "ADM1031",		pp_sensor_adm1031_ctor },
    { "ADS7830",		pp_sensor_ads7830_ctor },
    { "LM75",			pp_sensor_lm75_ctor },
    { "MAX6695",		pp_sensor_max6695_ctor },
    { "MAX6649",		pp_sensor_max6649_ctor },
    { "MAX1037ADC",		pp_sensor_max1037_adc_ctor },
    { "MAX6681",		pp_sensor_max6681_ctor },
    { "W83791DSens",		pp_sensor_w83791d_sens_ctor },
    { "ADM1029Sens",            pp_sensor_adm1029_sens_ctor },
    { "ADM1026Sens",            pp_sensor_adm1026_sens_ctor },
    { "ADM1024Sens",            pp_sensor_adm1024_sens_ctor },
    { "ADCKIRASens",            pp_sensor_adc_kira_ctor },
    { "W83792DSens",		pp_sensor_w83792d_sens_ctor },
    { "LM94Sens",		pp_lm94_sensor_ctor },
    { "W83793GSens",		pp_sensor_w83793g_sens_ctor },
    { "EMC6D103SSens",          pp_sensor_emc6d103s_sens_ctor },
    { "PCF8574sens",            pp_pcf8574_sensor_ctor },
    { "BLACKFORDsens",            pp_blackford_mch_sensor_ctor },
    { "ASUSGENERALSens",            pp_sensor_asusgeneral_sens_ctor },
    { "ASUSWritableSens",            pp_asus_sensor_asuswritable_ctor },
    { "ASUSPostInitMsgLedValue",            pp_asus_bmc_post_init_msgled_value_ctor },
    { "ADT7476",		pp_sensor_adt7476_ctor },
    { "ADT7476Sens",		pp_sensor_adt7476_sens_ctor },
    { "W83627HF",		pp_sensor_w83627hf_ctor },
    { "W83627HFSens",		pp_sensor_w83627hf_sens_ctor },
    { "W83627DHG",		pp_sensor_w83627dhg_ctor },
    { "W83627DHGSens",		pp_sensor_w83627dhg_sens_ctor },
    { "W83627DHG_1",		pp_sensor_w83627dhg_1_ctor },
    { "W83627DHGSens_1",	pp_sensor_w83627dhg_1_sens_ctor },
    { "W83627DHG_2",		pp_sensor_w83627dhg_2_ctor },
    { "W83627DHGSens_2",	pp_sensor_w83627dhg_2_sens_ctor },


    /* Conditions (these are sensors too!) */
    { "StaticCond",		pp_tp_static_cond_ctor },
    { "DelayedCond",            pp_tp_delayed_cond_ctor },
    { "SensHasValueCond",	pp_tp_sens_has_value_cond_ctor },
    { "I2CAliveCond",           pp_tp_i2c_alive_cond_ctor },
    { "AndCond",                pp_tp_and_cond_ctor },

    /* actors */
    { "GpioAct",		pp_tp_gpio_act_ctor },
    { "GpioMultiAct",           pp_tp_gpio_multi_act_ctor },
    { "FpgaPwm",		pp_fpga_pwm_dev_ctor },
    { "ADT7463pwm",             pp_adt7463_pwm_ctor },
    { "ADM1029PwmAct",          pp_sensor_adm1029_pwm_ctor },
    { "W83791DPwmAct",          pp_sensor_w83791d_pwm_ctor },
    { "W83792DClrIntrusion",	pp_sensor_w83792d_clr_intrusion_ctor },
    { "W83793GClrIntrusion",	pp_sensor_w83793g_clr_intrusion_ctor },
    
#ifdef LARA_KIMMSI
    { "MsiSmxAct",		pp_msi_smx_ctor },
    { "MsiFanControl",          pp_msi_fan_ctrl_ctor },
#endif

#ifdef PP_FEAT_OPMA_HW
    { "OpmaSmxAct",             pp_opma_smx_ctor },
#endif

#ifdef PRODUCT_SMIDC
    { "SupermicroSmxAct",       pp_smi_smx_ctor },
#endif
    { "DummySmx",           pp_dummy_smx_ctor },

#ifdef KIRA_RPC
    { "RCCY8Gpio",              pp_sensor_rccy8_gpio_ctor },

    { "RCCY8Sens",		pp_sensor_rccy8_sens_ctor},
    { "RPCCurrSens",		pp_rccy8_curr_sens_ctor},    
    { "MaxCurrSens",		pp_rccy8_max_curr_sens_ctor},    
    { "AptPowerSens",		pp_rccy8_apt_pwr_sens_ctor},
    { "RMSPowerSens",		pp_rccy8_rms_pwr_sens_ctor},
    { "RPCPwrSens",		pp_rccy8_pwr_sens_ctor},        
    { "MaxPowerSens",		pp_rccy8_max_pwr_sens_ctor},    
    { "CircuitBrkSens",		pp_sensor_rccy8_cbk_ctor},

    { "RCCY8DispAct",	        pp_rccy8_disp_act_ctor },

    { "RPCBeepCtrl",    pp_rpc_beep_ctrl_ctor },
    { "RPCLedCtrl",             pp_rccy8_led_ctrl_ctor },
    { "RPCDispCtrl",            pp_rccy8_disp_ctrl_ctor },
#endif /* KIRA_RPC */


    /* controllers */
    { "GpioLoopCtrl",		pp_tp_gpio_loop_ctor},
    { "EventCollectCtrl",	pp_tp_event_collect_ctrl_ctor },
    { "EventCollectPwmCtrl",	pp_tp_event_collect_pwm_ctrl_ctor },
    { "WarthogFanCtrl",		pp_warthog_fan_ctrl_ctor },
    { "CheyenneFanCtrl",	pp_cheyenne_fan_ctrl_ctor },
    { "LinearFanCtrl",          pp_linear_fan_ctrl_ctor },
    { "MuxedScanner",           pp_tp_muxed_scanner_ctor },    

    /* SDR configs */
    { "ThreshSDR",		pp_tp_thresh_sdr_ctor },
    { "DiscSDR",		pp_tp_disc_sdr_ctor },

    /* ASUS OEM configs */
    { "ASUSMBSupportMdl",		pp_asus_mb_support_model_ctor },
};

int pp_tp_ctors_basic_sz = sizeof(pp_tp_ctors_basic) / sizeof(pp_tp_obj_ctor_def_t);
