/*
 * <3BSD notice header>
 *
 * Copyright (C) 2003-2008 Raritan, Inc. <http://www.raritan.com/>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 * 
 *     * Neither the name of Raritan, Inc. nor the names of
 *       its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written
 *       permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <liberic_config.h>
#include <pp/intl.h>
#include <pp/ipmi.h>
#include "eric_base.h"
#include "eric_util.h"
#include "eric_forms.h"
#include "eric_form_vars.h"
#include "eric_validate.h"
#include "templates.h"

FV_SPEC = {
};

static int pre_validate_hook(webs_t wp, form_handler_t * fh);

int ipmi_chassis_control_tmpl_init(void)
{
    form_handler_t * fh;

    fh = CREATE_FH_INSTANCE(TEMPLATE_IPMI_CHASSIS_CONTROL, ACL_OBJ_POWER_CONTROL);
    fh->pre_validate_hook = pre_validate_hook;

    REGISTER_FH_INSTANCE_AND_RETURN(fh);
}

/* the formular handler (parses the buttons) */
static int pre_validate_hook(webs_t wp, form_handler_t * fh UNUSED)
{
    int cmd = -1;

    if (form_button_clicked(wp, "action_poweroff")) {
        cmd = PP_IPMI_CHASSIS_POWER_DOWN;
    } else if (form_button_clicked(wp, "action_poweron")) {
        cmd = PP_IPMI_CHASSIS_POWER_UP;
    } else if (form_button_clicked(wp, "action_powercycle")) {
        cmd = PP_IPMI_CHASSIS_POWER_CYCLE;
    } else if (form_button_clicked(wp, "action_softshutdown")) {
        cmd = PP_IPMI_CHASSIS_POWER_SOFT;
    } else if (form_button_clicked(wp, "action_reset")) {
        cmd = PP_IPMI_CHASSIS_RESET;
    } else if (form_button_clicked(wp, "action_diag")) {
        cmd = PP_IPMI_CHASSIS_POWER_DIAG;
    }

    if (cmd != -1) {
        const char *err;
        pp_log("Executing Power Control.\n");
        if (ipmi_chassis_power_control(wp, cmd, &err)) {
            set_response(wp, ERIC_RESPONSE_ERROR, err);
        } else {
            set_response(wp, ERIC_RESPONSE_OK, _("Successfully executed Power Control."));
        }
    }

    return 0;
}

/* backend for the buttons */
int ipmi_chassis_power_control(webs_t wp, int command, const char **errstr) {
    pp_ipmi_return_t ipmi_ret;
    pp_ipmi_parameter_t ipmi_param;
    int ret, err;

    if (errstr) *errstr = "";
    memset(&ipmi_ret, 0, sizeof(ipmi_ret));
    memset(&ipmi_param, 0, sizeof(ipmi_param));
    ipmi_param.data.chassis_power = command;

    ret = pp_ipmi_send_command(PP_IPMI_CMD_CHASSIS, PP_IPMI_CHASSIS_SUBCMD_POWER_CONTROL, &ipmi_param, &ipmi_ret, &err, wp->user);
    if (ret) {
        if (errstr) *errstr = pp_ipmi_get_error_string(err);
        pp_log("Error in sending Power Control command (%s).\n", errstr ? *errstr : pp_ipmi_get_error_string(err));
    }

    pp_ipmi_cleanup_ret(&ipmi_ret);
    return ret;
}
